<?php

if (!defined('BASEPATH'))
    exit('No direct script access allowed');

class Announcements extends Admin_Controller
{

    public function __construct()
    {
        parent::__construct();
        $this->load->model('announcements_model');
        $this->load->model('items_model');
        

    }

    public function index($id = NULL)
    {

        $data['title'] = lang('all') . ' ' . lang('announcements');
        if ($id) {
            $data['announcements'] = $this->db->where('announcements_id', $id)->get('tbl_announcements')->row();
            if (empty($data['announcements'])) {
                $type = "error";
                $message = "No Record Found";
                set_message($type, $message);
                redirect('admin/announcements');
            }
        }
        if(!empty($id))
        {
            $data['active'] = 2;
            if(!empty($data['announcements']->state_id))
            {
                // get districts by state id
                $data['districts'] = $this->items_model->getAllDistrictsByStateId($data['announcements']->state_id);
            }
        }
        else
        {
            $data['active'] = 1;
            $data['all_announcements'] = $this->db->get('tbl_announcements')->result();
        }        
        $data['dropzone'] = true;
        $data['states'] = $this->announcements_model->get_all_states();
        $data['subview'] = $this->load->view('admin/announcements/all_announcements', $data, TRUE);
        $this->load->view('admin/_layout_main', $data); //page load
    }

    public function new_announcements($id = null)
    {
        $data['title'] = lang('new') . ' ' . lang('announcements'); //Page title

        if (!empty($id)) {
            $edited = can_action('100', 'edited');
            if (!empty($edited)) {
                $data['announcements'] = $this->db->where('announcements_id', $id)->get('tbl_announcements')->row();
            }
            if (empty($data['announcements'])) {
                $type = "error";
                $message = "No Record Found";
                set_message($type, $message);
                redirect('admin/announcements/create_announcements');
            }
        }
        $data['subview'] = $this->load->view('admin/announcements/new_announcements', $data, FALSE);
        $this->load->view('admin/_layout_modal_lg', $data); //page load
    }

    public function announcementsList()
    {
        if ($this->input->is_ajax_request()) {
            $this->load->model('datatables');
            $this->datatables->table = 'tbl_announcements';
            $this->datatables->join_table = array('	tbl_account_details');
            $this->datatables->join_where = array('tbl_account_details.user_id=tbl_announcements.user_id');
            $this->datatables->column_order = array('title', 'description', 'start_date', 'end_date', 'tbl_account_details.fullname',);
            $this->datatables->column_search = array('title', 'description', 'start_date', 'end_date', 'tbl_account_details.fullname',);
            $this->datatables->order = array('announcements_id' => 'desc');

            $fetch_data = make_datatables();

            $edited = can_action('100', 'edited');
            $deleted = can_action('100', 'deleted');
            $data = array();
            foreach ($fetch_data as $_key => $v_announcements) {
                $action = null;
                $sub_array = array();
                $sub_array[] = $v_announcements->title;
                $sub_array[] = fullname($v_announcements->user_id);
                if($v_announcements->all_client==1)
                {
                    $share_with = 'All Customers';
                }
                else if($v_announcements->all_client==2)
                {
                    $share_with = 'Registered Customers';
                }
                else if($v_announcements->all_client==3)
                {
                    $share_with = 'Existing Customers';
                }
                else if($v_announcements->all_client==4)
                {
                    $share_with = 'Selected From State/District';
                }
                $sub_array[] = $share_with;
                $state_name = '-';
                $district_name = '-';
                if(!empty($v_announcements->state_id))
                {
                    $state = $this->db->where('id', $v_announcements->state_id)->get('tbl_states')->row();
                    $state_name = $state->name;
                }
                if(!empty($v_announcements->district_id))
                {
                    $district = $this->db->where('id', $v_announcements->district_id)->get('tbl_districts')->row();
                    $district_name = $district->name;
                }
                $sub_array[] = $state_name;
                $sub_array[] = $district_name;

                
                /*$sub_array[] = display_date($v_announcements->start_date);
                $sub_array[] = display_date($v_announcements->end_date);*/
                $custom_form_table = custom_form_table(16, $v_announcements->announcements_id);
                if (!empty($custom_form_table)) {
                    foreach ($custom_form_table as $c_label => $v_fields) {
                        $sub_array[] = $v_fields;
                    }
                }
                if ($v_announcements->status == 'unpublished') {
                    $status = '<span class="label label-danger">' . lang('unpublished') . ' </span>';
                } else {
                    $status = '<span class="label label-success"> ' . lang('published') . ' </span>';
                }
                $sub_array[] = $status;

                $action .= btn_view_modal('admin/announcements/announcements_details/' . $v_announcements->announcements_id) . ' ';
                if (!empty($edited) && $v_announcements->status == 'unpublished') {
                    $action .= '<a href="' . base_url() . 'admin/announcements/index/' . $v_announcements->announcements_id . '"
                               class="btn btn-primary btn-xs" title="' . lang('edit') . '"><span class="fa fa-pencil-square-o"></span></a>  ';
                }
                if (!empty($deleted)) {
                    $action .= ajax_anchor(base_url('admin/announcements/delete_announcements/' . $v_announcements->announcements_id), "<i class='btn btn-xs btn-danger fa fa-trash-o'></i>", array("class" => "", "title" => lang('delete'), "data-fade-out-on-success" => "#table_" . $_key)) . ' ';
                }

                $sub_array[] = $action;
                $data[] = $sub_array;
            }

            render_table($data);
        } else {
            redirect('admin/dashboard');
        }
    }

    public function save_announcements($id = NULL)
    {
        $created = can_action('100', 'created');
        $edited = can_action('100', 'edited');
        if (!empty($created) || !empty($edited) && !empty($id)) {
            $data = $this->announcements_model->array_from_post(array(
                'title',
                'description',
                'start_date',
                'end_date',
                'all_client',
                'status',
                'state_id',
                'district_id'
            ));

            $upload_file = array();
            $files = $this->input->post("files", true);
            $target_path = getcwd() . "/uploads/";
            //process the fiiles which has been uploaded by dropzone

            if (!empty($files) && is_array($files)) {
                foreach ($files as $key => $file) {
                    if (!empty($file)) {
                        $file_name = $this->input->post('file_name_' . $file, true);

                        $new_file_name = move_temp_file($file_name, $target_path);
                        $file_ext = explode(".", $new_file_name);
                        $is_image = check_image_extension($new_file_name);
                        $size = $this->input->post('file_size_' . $file, true) / 1000;
                        if ($new_file_name) {
                            $up_data = array(
                                "fileName" => $new_file_name,
                                "path" => "uploads/" . $new_file_name,
                                "fullPath" => getcwd() . "/uploads/" . $new_file_name,
                                "ext" => '.' . end($file_ext),
                                "size" => round($size, 2),
                                "is_image" => $is_image,
                            );
                            array_push($upload_file, $up_data);
                        }
                    }
                }
            }

            $fileName = $this->input->post('fileName', true);
            $path = $this->input->post('path', true);
            $fullPath = $this->input->post('fullPath', true);
            $size = $this->input->post('size', true);
            $is_image = $this->input->post('is_image', true);
            if(!empty($path))
            {
                unlink($path);
                $fileName = '';
            }
            if (!empty($fileName)) {
                foreach ($fileName as $key => $name) {
                    $old['fileName'] = $name;
                    $old['path'] = $path[$key];
                    $old['fullPath'] = $fullPath[$key];
                    $old['size'] = $size[$key];
                    $old['is_image'] = $is_image[$key];

                    array_push($upload_file, $old);
                }
            }
            //echo '<pre>';print_r($upload_file);die;
            if (!empty($upload_file)) {
                $data['attachment'] = json_encode($upload_file);
            } else {
                $data['attachment'] = null;
            }

            $data['user_id'] = $this->session->userdata('user_id');

            $this->announcements_model->_table_name = "tbl_announcements"; // table name
            $this->announcements_model->_primary_key = "announcements_id"; // $id
            $return_id = $this->announcements_model->save($data, $id);

            save_custom_field(16, $return_id);

            if (!empty($id)) {
                $activity = 'activity_update_announcements';
                $msg = lang('announcements_information_update');
            } else {
                $activity = 'activity_added_announcements';
                $msg = lang('announcements_information_saved');

                $this->items_model->_table_name = "tbl_leads"; // table name
                $this->items_model->_primary_key = "leads_id"; 
                
                if($this->input->post('status') == 'published')
                {
                    if ($data['all_client'] == 1) 
                    {   
                        // send notification to all customers
                        $this->items_model->_table_name = "tbl_client"; // table name
                        $this->items_model->_primary_key = "client_id";
                        $client_data = $this->items_model->get();

                        $this->items_model->_table_name = "tbl_leads"; // table name
                        $this->items_model->_primary_key = "leads_id";
                        $lead_data = $this->items_model->get();
                        $leads_data = array_merge($client_data, $lead_data);
                    } 
                    else if ($data['all_client'] == 2) 
                    {
                        // send notification to registered customers from leads
                        $leads_data = $this->items_model->get();
                    }
                    else if ($data['all_client'] == 3) 
                    {
                        // send notification to all existing customers
                        $this->items_model->_table_name = "tbl_client"; // table name
                        $this->items_model->_primary_key = "client_id";
                        $leads_data = $this->items_model->get();
                    }
                    else if ($data['all_client'] == 4)
                    {
                        // send notification to all customers from state/district
                        $state_id = $this->input->post('state_id');
                        $district_id = $this->input->post('district_id') ? $this->input->post('district_id') : NULL;
                        if(!empty($state_id))
                        {
                            $where = array('state_id' => $state_id, 'district_id' => $district_id);
                            $leads_data = $this->items_model->get_by($where,FALSE);
                        }
                    } 
                    //echo '<pre>';print_r($leads_data);die;
                    // send notification to all leads             
                    $message = $this->input->post('title');
                    $type = 'Announcement';
                    if(!empty($leads_data))
                    {
                        foreach($leads_data as $lead)
                        {
                            if(!empty($lead->notification_id))
                            {
                                $responce = $this->sendNotification($lead->notification_id,$message,$type,$return_id);
                            }
                        }
                    }
                }                
            }
            // save into activities
            $activities = array(
                'user' => $this->session->userdata('user_id'),
                'module' => 'announcements',
                'module_field_id' => $id,
                'activity' => $activity,
                'icon' => 'fa-ticket',
                'value1' => $data['title'],
            );

            // Update into tbl_project
            $this->announcements_model->_table_name = "tbl_activities"; //table name
            $this->announcements_model->_primary_key = "activities_id";
            $this->announcements_model->save($activities);

            // messages for user
            $type = "success";
            $message = $msg;
            set_message($type, $message);
        }
        redirect('admin/announcements');
    }

    public function download_files($id, $fileName)
    {
        $appl_info = $this->announcements_model->check_by(array('announcements_id' => $id), 'tbl_announcements');
        if (empty($appl_info)) {
            $type = "error";
            $message = "No Record Found";
            set_message($type, $message);
            redirect('admin/announcements');
        }
        $this->load->helper('download');
        if (!empty($appl_info->attachment)) {
            $down_data = file_get_contents('uploads/' . $fileName); // Read the file's contents
            force_download($fileName, $down_data);
        }
    }

    public function announcements_details($id)
    {
        $data['title'] = lang('announcements_details'); //Page title

        $this->announcements_model->_table_name = "tbl_announcements"; // table name
        $this->announcements_model->_order_by = "announcements_id"; // $id
        $data['announcements_details'] = $this->announcements_model->get_by(array('announcements_id' => $id), TRUE);
        if (empty($data['announcements_details'])) {
            $type = "error";
            $message = "No Record Found";
            set_message($type, $message);
            redirect('admin/announcements');
        }
        $this->announcements_model->_primary_key = 'announcements_id';
        $updata['view_status'] = '1';
        $this->announcements_model->save($updata, $id);
        $data['subview'] = $this->load->view('admin/announcements/announcements_details', $data, FALSE);
        $this->load->view('admin/_layout_modal_lg', $data); //page load
    }

    public function view_announcements_details($id)
    {
        $data['title'] = lang('announcements_details'); //Page title

        $this->announcements_model->_table_name = "tbl_announcements"; // table name
        $this->announcements_model->_order_by = "announcements_id"; // $id
        $data['announcements_details'] = $this->announcements_model->get_by(array('announcements_id' => $id), TRUE);
        if (empty($data['announcements_details'])) {
            $type = "error";
            $message = "No Record Found";
            set_message($type, $message);
            redirect('admin/announcements');
        }
        $this->announcements_model->_primary_key = 'announcements_id';
        $updata['view_status'] = '1';
        $this->announcements_model->save($updata, $id);
        $data['subview'] = $this->load->view('admin/announcements/announcements_details', $data, TRUE);
        $this->load->view('admin/_layout_main', $data); //page load
    }

    public function delete_announcements($id = NULL)
    {
        $deleted = can_action('100', 'deleted');
        if (!empty($deleted)) {
            $announcements_info = $this->announcements_model->check_by(array('announcements_id' => $id), 'tbl_announcements');
            if (empty($announcements_info)) {
                $type = "error";
                $message = "No Record Found";
                set_message($type, $message);
                redirect('admin/announcements');
            }
            // save into activities
            $activities = array(
                'user' => $this->session->userdata('user_id'),
                'module' => 'announcements',
                'module_field_id' => $id,
                'activity' => 'activity_delete_announcements',
                'icon' => 'fa-ticket',
                'value1' => $announcements_info->title,
            );

            // Update into tbl_project
            $this->announcements_model->_table_name = "tbl_activities"; //table name
            $this->announcements_model->_primary_key = "activities_id";
            $this->announcements_model->save($activities);

            $this->announcements_model->_table_name = "tbl_announcements";
            $this->announcements_model->_primary_key = "announcements_id";
            $this->announcements_model->delete($id);;

            // messages for user
            $type = "success";
            $message = lang('announcements_information_delete');
            set_message($type, $message);
        }
        redirect('admin/announcements');
    }

}
