<?php

if (!defined('BASEPATH'))
    exit('No direct script access allowed');

class Dataleads extends Admin_Controller
{

    public function __construct()
    {
        parent::__construct();
        $this->load->model('items_model');
        $this->load->model('client_model');
        $this->load->library('form_validation');
        $this->load->helper('ckeditor');
        $this->data['ckeditor'] = array(
            'id' => 'ck_editor',
            'path' => 'asset/js/ckeditor',
            'config' => array(
                'toolbar' => "Full",
                'width' => "99.8%",
                'height' => "400px"
            )
        );
    }

    public function index($id = NULL)
    {
        $data['title'] = lang('all_data_leads');
        $data['active'] = 1;
        $data['assign_user'] = $this->items_model->allowed_user('162');

        $data['states'] = $this->items_model->get_all_states();
        $data['categories'] = $this->items_model->get_all_lead_categories();
        $data['products'] = $this->items_model->get_all_products();

        $data['subview'] = $this->load->view('admin/data_leads/all_data_leads', $data, TRUE);
        $this->load->view('admin/_layout_main', $data); //page load
    }

    public function dataleadList($filterBy = null, $search_by = null)
    {
        if ($this->input->is_ajax_request()) {
            $this->load->model('datatables');
            $this->datatables->table = 'tbl_leads';
            $custom_field = custom_form_table_search(5);
            $main_column = array('CONCAT(first_name," ",middle_name," ",last_name) as lead_name', 'mobile', 'city', 'address','lead_status');
            $action_array = array('leads_id');
            $result = array_merge($main_column, $custom_field, $action_array);
            $this->datatables->column_order = $result;
            $this->datatables->column_search = $result;
            $this->datatables->order = array('leads_id' => 'ASC');

            $where = array('type' => 'Data Lead');
            $fetch_data = $this->datatables->get_leads($filterBy, $search_by ,$where);

            $data = array();

            $edited = can_action('162', 'edited');
            $deleted = can_action('162', 'deleted');
            
            foreach ($fetch_data as $_key => $v_leads) {
                $action = null;
                if (!empty($v_leads)) {
                    $sub_array = array();
                    if (!empty($deleted)) {
                        $sub_array[] = '<div class="checkbox c-checkbox" ><label class="needsclick"> <input value="' . $v_leads->leads_id . '" type="checkbox"><span class="fa fa-check"></span></label></div>';
                    }
                    $name = null;
                    $name .= '<a class="text-info" href="' . base_url() . 'admin/leads/leads_details/' . $v_leads->leads_id . '">' . $v_leads->first_name . ' ' .$v_leads->middle_name . ' ' . $v_leads->last_name . '</a>';

                    $sub_array[] = $name;
                    $sub_array[] = $v_leads->mobile;
                    $sub_array[] = $v_leads->city;
                    $sub_array[] = $v_leads->address;
                    $sub_array[] = $v_leads->lead_status;

                   
                    if (!empty($edited)) {
                        $action .= btn_edit('admin/dataleads/edit/' . $v_leads->leads_id) . ' ';
                    }
                    if (!empty($deleted)) {
                        $action .= ajax_anchor(base_url("admin/leads/delete_leads/$v_leads->leads_id"), "<i class='btn btn-xs btn-danger fa fa-trash-o'></i>", array("class" => "", "title" => lang('delete'), "data-fade-out-on-success" => "#table_" . $_key)) . ' ';
                    }
                    $sub_array[] = $action;
                    $data[] = $sub_array;
                }

            }

            render_table($data, $where);
        } else {
            redirect('admin/dashboard');
        }
    }

    
    

    public function saved_dataleads($id = NULL) 
    {  
        $created = can_action('55', 'created');
        $edited = can_action('55', 'edited');
        if (!empty($created) || !empty($edited)) {
            $this->items_model->_table_name = 'tbl_leads';
            $this->items_model->_primary_key = 'leads_id';

            $this->form_validation->set_rules('first_name', lang('first_name'),'trim|required');
            $this->form_validation->set_rules('last_name', lang('last_name'),'trim|required');
            $this->form_validation->set_rules('mobile', lang('mobile'),'trim|required');
            $this->form_validation->set_rules('address', lang('address'),'trim|required');

            if($this->form_validation->run() == true)
            {
                $data = $this->items_model->array_from_post(array('first_name', 'middle_name', 'last_name', 'email', 'mobile', 'address', 'state_id', 'district_id', 'city', 'pincode', 'lead_category_id', 'lead_subcategory_id', 'product_id', 'calculation_type', 'consumer_number', 'supplier', 'units', 'approx_unit_rate', 'unit_type', 'no_of_months', 'lead_source_id', 'lead_type', 'reference'));

                $data['lead_status'] = 'New';
                $data['type'] = 'Data Lead';
                $data['created_by'] = $this->session->userdata('user_id');
                if($this->input->post('lead_source') == 'Own')
                {
                    $data['referred_by'] = $this->session->userdata('user_id');
                }
                $where = array('mobile' => $data['mobile'], 'first_name' => $data['first_name'], 'last_name' => $data['last_name']);
                $check_leads = $this->items_model->check_update('tbl_leads', $where);
                if (!empty($check_leads)) 
                { 
                    $type = 'error';
                    $msg = "<strong style='color:#000'>" . $data['lead_name'] . '</strong>  ' . lang('already_exist');
                } 
                else 
                { 
                    if(!empty($id))
                    {
                        $lead = $this->items_model->check_by(array('leads_id' => $id), 'tbl_leads');
                        if($lead->calculation_type!=$this->input->post('calculation_type'))
                        {
                            if($this->input->post('calculation_type') == 'unit')
                            {
                                if(is_file($lead->attachment))
                                {
                                    unlink($lead->attachment);
                                }
                                $data['attachment'] = '';
                            }
                            else if($this->input->post('calculation_type') == 'fileupload')
                            {
                                $data['consumer_number'] = '';
                                $data['supplier'] = '';
                                $data['units'] = '';
                                $data['approx_unit_rate'] = '';
                                $data['unit_type'] = '';
                                $data['no_of_months'] = '';
                            }
                        }
                    }

                    if($this->input->post('calculation_type') == 'fileupload')
                    {
                        if (isset($_FILES['attachment']) && $_FILES['attachment']['name'] != '') 
                        {
                            if($lead->calculation_type == 'fileupload')
                            {
                                if(is_file($lead->attachment))
                                {
                                    unlink($lead->attachment);
                                }
                            }

                            $file_path = './uploads/lead_attachments/';
                            if (!is_dir($file_path)) 
                            {
                                mkdir($file_path, 0777, TRUE);
                            }
                            $date = date('Y-m-d H:i:s');
                            $name = strtotime($date);
                            $ext = pathinfo(basename( $_FILES['attachment']['name']), PATHINFO_EXTENSION);
                            $act_file_path = $file_path . $name.'.'.$ext;
                            move_uploaded_file($_FILES['attachment']['tmp_name'], $act_file_path);
                            $data['attachment'] = $act_file_path;
                        }
                    }

                    $return_id = $this->items_model->save($data, $id);
                    if($return_id)
                    {
                        if($this->input->post('state_id') && $this->input->post('district_id'))
                        {
                            $year= date("Y");
                            $statePadded = sprintf("%02d", $this->input->post('state_id'));
                            $districtPadded = sprintf("%04d",$this->input->post('district_id'));
                            $idPadded = sprintf("%06d", $return_id);
                            $code=$year.$statePadded.$districtPadded.$idPadded;
                            $this->db->where('leads_id',$return_id);
                            $this->db->update('tbl_leads',array('lead_number' => $code));
                        }
                    }
                
                    $id = $return_id;
                    $action = 'activity_save_data_leads';
                    $description = 'not_save_data_leads';
                    $msg = lang('save_leads');                    
                
                    $activity = array(
                        'user' => $this->session->userdata('user_id'),
                        'module' => 'data_leads',
                        'module_field_id' => $id,
                        'activity' => $action,
                        'icon' => 'fa-rocket',
                        'link' => 'admin/dataleads/leads_details/' . $id,
                        'value1' => $data['first_name'] . ' ' . $data['middle_name'] . ' ' . $data['last_name']
                    );
                    $this->items_model->_table_name = 'tbl_activities';
                    $this->items_model->_primary_key = 'activities_id';
                    $this->items_model->save($activity);
                    // messages for user
                    $type = "success";
                    $message = $msg;
                    set_message($type, $message);
                    redirect('admin/dataleads');
                } // end of else
            } // end of form validation

            if($this->form_validation->run() == false || empty($return_id))
            {
                $data['error'] = (validation_errors() ? validation_errors() : ($error ? $error : ''));
                $data['title'] = lang('leads');
                $data['active'] = 2;
                $data['districts'] = $this->items_model->getAllDistrictsByStateId($this->input->post('state_id'));
                $data['subcategories'] = $this->items_model->getAllSubcategoriesByCatId($this->input->post('lead_category_id'));
                $data['states'] = $this->items_model->get_all_states();
                $data['categories'] = $this->items_model->get_all_lead_categories();
                $data['products'] = $this->items_model->get_all_products();

                $data['subview'] = $this->load->view('admin/data_leads/all_data_leads', $data, TRUE);
                $this->load->view('admin/_layout_main', $data);
            }
        }
        
    }

    public function edit($id)
    {
        $data['title'] = lang('edit_data_leads');
        $data['active'] = 1;
        $data['leads_info'] = $this->items_model->check_by(array('leads_id' => $id), 'tbl_leads');
        $data['districts'] = $this->items_model->getAllDistrictsByStateId($data['leads_info']->state_id);
        $data['subcategories'] = $this->items_model->getAllSubcategoriesByCatId($data['leads_info']->lead_category_id);
                
        $data['assign_user'] = $this->items_model->allowed_user('162');
        $data['states'] = $this->items_model->get_all_states();
        $data['categories'] = $this->items_model->get_all_lead_categories();
        $data['products'] = $this->items_model->get_all_products();

        $data['subview'] = $this->load->view('admin/data_leads/edit_data_leads', $data, TRUE);
        $this->load->view('admin/_layout_main', $data); //page load
    }

    // Update data lead
    public function update_leads($id = NULL) 
    {
        $created = can_action('162', 'created');
        $edited = can_action('162', 'edited');
        if (!empty($created) || !empty($edited) && !empty($id)) 
        {
            $this->items_model->_table_name = 'tbl_leads';
            $this->items_model->_primary_key = 'leads_id';

            $this->form_validation->set_rules('first_name', lang('first_name'),'trim|required');
            $this->form_validation->set_rules('last_name', lang('last_name'),'trim|required');
            $this->form_validation->set_rules('mobile', lang('mobile'),'trim|required');
            $this->form_validation->set_rules('address', lang('address'),'trim|required');
            $this->form_validation->set_rules('state_id', lang('state'),'trim|required');
            $this->form_validation->set_rules('district_id', lang('district'),'trim|required');
            $this->form_validation->set_rules('city', lang('city'),'trim|required');
            $this->form_validation->set_rules('pincode', lang('pincode'),'trim|required');
            $this->form_validation->set_rules('lead_category_id', lang('lead_categories'),'trim|required');
            $this->form_validation->set_rules('product_id', lang('product'),'trim|required');
            $this->form_validation->set_rules('units', lang('units'),'trim|required');
            $this->form_validation->set_rules('approx_unit_rate', lang('approximate_unit_rate'),'trim|required'); 
            $this->form_validation->set_rules('unit_type', lang('unit_type'),'trim|required'); 
            $this->form_validation->set_rules('lead_type', lang('lead_type'),'trim|required');  

            if($this->input->post('unit_type')=='Monthly')
            {
                $this->form_validation->set_rules('no_of_months', lang('no_of_months'),'trim|required');
            } // end if
            
            if($this->form_validation->run() == true)
            {
                $data = $this->items_model->array_from_post(array('first_name', 'middle_name', 'last_name', 'email', 'mobile', 'address', 'state_id', 'district_id', 'city', 'pincode', 'lead_category_id', 'lead_subcategory_id', 'product_id', 'calculation_type', 'consumer_number', 'supplier', 'units', 'approx_unit_rate', 'unit_type', 'no_of_months', 'lead_type'));

                $data['rescheduled_time'] = NULL;
                $data['rescheduled_reason'] = NULL;
                $data['updated_by'] = $this->session->userdata('user_id');
                $data['updated_date'] = date('Y-m-d H:i:s');
                $data['calculation_type'] = 'unit';
                $data['lead_status'] = 'New';
                if($this->input->post('lead_type') == 'Hot')
                {
                    $data['type'] = 'Lead';
                    $data['lead_status'] = 'Ready To Assign';
                }
                if(!empty($id))
                {
                    $lead = $this->items_model->check_by(array('leads_id' => $id), 'tbl_leads');
                }

                if (isset($_FILES['attachment']) && $_FILES['attachment']['name'] != '') 
                {
                    if($lead->calculation_type == 'fileupload')
                    {
                        if(is_file($lead->attachment))
                        {
                            unlink($lead->attachment);
                        } //end if
                    }

                    $file_path = './uploads/lead_attachments/';
                    if (!is_dir($file_path)) 
                    {
                        mkdir($file_path, 0777, TRUE);
                    }
                    $date = date('Y-m-d H:i:s');
                    $name = strtotime($date);
                    $ext = pathinfo(basename( $_FILES['attachment']['name']), PATHINFO_EXTENSION);
                    $act_file_path = $file_path . $name.'.'.$ext;
                    move_uploaded_file($_FILES['attachment']['tmp_name'], $act_file_path);
                    $data['attachment'] = $act_file_path;
                }

                $return_id = $this->items_model->save($data, $id);
                if($return_id)
                {
                    $year= date("Y");
                    $statePadded = sprintf("%02d", $this->input->post('state_id'));
                    $districtPadded = sprintf("%04d",$this->input->post('district_id'));
                    $idPadded = sprintf("%06d", $return_id);
                    $code=$year.$statePadded.$districtPadded.$idPadded;
                    $this->db->where('leads_id',$return_id);
                    $this->db->update('tbl_leads',array('lead_number' => $code));

                    // code for added entry in avtivity table
                    if (!empty($id)) 
                    {
                        $id = $id;
                        $action = 'activity_update_data_leads';
                        $description = 'not_update_data_leads';
                        $msg = lang('update_data_leads');
                    }

                    $activity = array(
                        'user' => $this->session->userdata('user_id'),
                        'module' => 'leads',
                        'module_field_id' => $id,
                        'activity' => $action,
                        'icon' => 'fa-rocket',
                        'link' => 'admin/leads/leads_details/' . $id,
                        'value1' => $data['first_name'] . ' ' . $data['middle_name'] . ' ' . $data['last_name']
                    );
                    $this->items_model->_table_name = 'tbl_activities';
                    $this->items_model->_primary_key = 'activities_id';
                    $this->items_model->save($activity);

                    // messages for user
                    $type = "success";
                    $message = $msg;
                    set_message($type, $message);
                    if($this->input->post('lead_type') == 'Hot')
                    {
                        redirect('admin/dataleads');
                    }
                    else
                    {
                        redirect('admin/dataleads/edit/'.$id);
                    }
                                      
                }                
            } // end of validation if

            if($this->form_validation->run() == false || empty($return_id) || empty($clientid))
            {
                $data['error'] = (validation_errors() ? validation_errors() : ($error ? $error : ''));
                $data['title'] = lang('leads');
                $data['active'] = 1;
                $data['leads_info'] = $this->items_model->check_by(array('leads_id' => $id), 'tbl_leads');
                $data['districts'] = $this->items_model->getAllDistrictsByStateId($data['leads_info']->state_id);
                $data['subcategories'] = $this->items_model->getAllSubcategoriesByCatId($data['leads_info']->lead_category_id);
                $data['states'] = $this->items_model->get_all_states();
                $data['categories'] = $this->items_model->get_all_lead_categories();
                $data['products'] = $this->items_model->get_all_products();

                $data['subview'] = $this->load->view('admin/data_leads/edit_data_leads', $data, TRUE);
                $this->load->view('admin/_layout_main', $data);
            }
        } // end of edited if
    } //  end of function

    // update status for call center
    public function update_status($id = NULL)  
    {  
        $edited = can_action('162', 'edited');
        if (!empty($edited) && !empty($id)) {
            $this->form_validation->set_rules('lead_status', lang('lead_status'),'trim|required');
            $this->form_validation->set_rules('comment', lang('comment'),'trim|required');
            if($this->form_validation->run() == true)
            {
                $this->items_model->_table_name = 'tbl_leads';
                $this->items_model->_primary_key = 'leads_id';
                $data = $this->items_model->array_from_post(array('lead_status', 'comment'));
                $data['updated_by'] = $this->session->userdata('user_id');
                $data['updated_date'] = date('Y-m-d H:i:s');
                
                $return_id = $this->items_model->save($data, $id);
                if($return_id)
                {
                    if (!empty($id)) 
                    {
                        $id = $id;
                        $action = 'activity_update_data_leads';
                        $description = 'not_update_data_leads';
                        $msg = lang('update_data_leads_status');
                    } 

                    $activity = array(
                        'user' => $this->session->userdata('user_id'),
                        'module' => 'leads',
                        'module_field_id' => $id,
                        'activity' => $action,
                        'icon' => 'fa-rocket',
                        'link' => 'admin/dataleads/leads_details/' . $id
                    );
                    $this->items_model->_table_name = 'tbl_activities';
                    $this->items_model->_primary_key = 'activities_id';
                    $this->items_model->save($activity);
                    // messages for user
                    $type = "success";            
                    $message = $msg;
                    set_message($type, $message);
                    redirect('admin/dataleads');
                }
                else
                {
                    $error = lang('update_data_lead_error');
                }
            }

            if($this->form_validation->run() == false || empty($return_id))
            {
                $data['error'] = (validation_errors() ? validation_errors() : ($error) ? $error : '');
                $data['title'] = lang('edit_data_leads');
                $data['active'] = 3;
                $data['leads_info'] = $this->items_model->check_by(array('leads_id' => $id), 'tbl_leads');
                $data['districts'] = $this->items_model->getAllDistrictsByStateId($data['leads_info']->state_id);
                $data['subcategories'] = $this->items_model->getAllSubcategoriesByCatId($data['leads_info']->lead_category_id);
                $data['states'] = $this->items_model->get_all_states();
                $data['categories'] = $this->items_model->get_all_lead_categories();
                $data['products'] = $this->items_model->get_all_products();

                $data['subview'] = $this->load->view('admin/data_leads/edit_data_leads', $data, TRUE);
                $this->load->view('admin/_layout_main', $data);
            }            
        }        
    }   
    
    // update status for call center
    public function reschedule_leads($id = NULL)  
    {  
        $edited = can_action('162', 'edited');
        if (!empty($edited) && !empty($id)) {
            $this->form_validation->set_rules('rescheduled_time', lang('reschedule_time'),'trim|required');
            $this->form_validation->set_rules('rescheduled_reason', lang('reschedule_reason'),'trim|required');
            if($this->form_validation->run() == true)
            {
                $this->items_model->_table_name = 'tbl_leads';
                $this->items_model->_primary_key = 'leads_id';
                $data = $this->items_model->array_from_post(array('rescheduled_time', 'rescheduled_reason'));
                $data['lead_status'] = 'Rescheduled';
                $data['updated_by'] = $this->session->userdata('user_id');
                $data['updated_date'] = date('Y-m-d H:i:s');
                $return_id = $this->items_model->save($data, $id);
                if($return_id)
                {
                    if (!empty($id)) 
                    {
                        $id = $id;
                        $action = 'activity_update_data_leads';
                        $description = 'not_update_data_leads';
                        $msg = lang('update_data_leads');
                    } 

                    $activity = array(
                        'user' => $this->session->userdata('user_id'),
                        'module' => 'leads',
                        'module_field_id' => $id,
                        'activity' => $action,
                        'icon' => 'fa-rocket',
                        'link' => 'admin/dataleads/leads_details/' . $id,
                    );
                    $this->items_model->_table_name = 'tbl_activities';
                    $this->items_model->_primary_key = 'activities_id';
                    $this->items_model->save($activity);
                    // messages for user
                    $type = "success";            
                    $message = $msg;
                    set_message($type, $message);
                    redirect('admin/dataleads');
                }
                else
                {
                    $error = lang('update_data_lead_error');
                }
            }

            if($this->form_validation->run() == false || empty($return_id))
            {
                $data['error'] = (validation_errors() ? validation_errors() : ($error ? $error : ''));
                $data['title'] = lang('edit_data_leads');
                $data['active'] = 2;
                $data['leads_info'] = $this->items_model->check_by(array('leads_id' => $id), 'tbl_leads');
                $data['districts'] = $this->items_model->getAllDistrictsByStateId($data['leads_info']->state_id);
                $data['subcategories'] = $this->items_model->getAllSubcategoriesByCatId($data['leads_info']->lead_category_id);
                $data['states'] = $this->items_model->get_all_states();
                $data['categories'] = $this->items_model->get_all_lead_categories();
                $data['products'] = $this->items_model->get_all_products();

                $data['subview'] = $this->load->view('admin/data_leads/edit_data_leads', $data, TRUE);
                $this->load->view('admin/_layout_main', $data);
            }            
        }        
    }

    public function import_data_leads()
    {
        $data['title'] = lang('import_dataleads');
        $data['assign_user'] = $this->items_model->allowed_user('162');
        // get all leads status

        $data['subview'] = $this->load->view('admin/data_leads/import_dataleads', $data, TRUE);
        $this->load->view('admin/_layout_main', $data); //page load
    }
      
    // data leads imported by excel 
    public function save_imported()
    {
        //load the excel library
        $this->load->library('excel');
        ob_start();
        $file = $_FILES["upload_file"]["tmp_name"];
        if (!empty($file)) {
            $valid = false;
            $types = array('Excel2007', 'Excel5','CSV');
            foreach ($types as $type) {
                $reader = PHPExcel_IOFactory::createReader($type);
                if ($reader->canRead($file)) {
                    $valid = true;
                }
            }
            if (!empty($valid)) {
                try {
                    $objPHPExcel = PHPExcel_IOFactory::load($file);
                } catch (Exception $e) {
                    die("Error loading file :" . $e->getMessage());
                }
                //All data from excel
                $sheetData = $objPHPExcel->getActiveSheet()->toArray(null, true, true, true);
                $invalidcnt = 0;
                $duplicatecnt = 0;
                $duplicate_data = array();
                for ($x = 2; $x <= count($sheetData); $x++) {
                    // **********************
                    // Save Into leads table
                    // **********************
                    $data = $this->items_model->array_from_post(array('first_name', 'middle_name', 'last_name', 'email', 'mobile', 'address', 'city', 'pincode'));
                    if(!empty($sheetData[$x]["A"]) && !empty($sheetData[$x]["C"] && $sheetData[$x]["E"]))
                    {
                        $data['first_name'] = trim($sheetData[$x]["A"]);
                        $data['middle_name'] = trim($sheetData[$x]["B"]);
                        $data['last_name'] = trim($sheetData[$x]["C"]);
                        $data['email'] = trim($sheetData[$x]["D"]);
                        $data['mobile'] = trim($sheetData[$x]["E"]);
                        $data['address'] = trim($sheetData[$x]["F"]);
                        $data['city'] = trim($sheetData[$x]["G"]);
                        $data['pincode'] = trim($sheetData[$x]["H"]);
                    
                        $data['lead_source_id'] = $this->input->post('lead_source_id');
                        $data['lead_status'] = 'New';
                        $data['type'] = 'Data Lead';
                        $data['created_by'] = $this->session->userdata('user_id');
                        if($this->input->post('lead_source') == 'Own')
                        {
                            $data['referred_by'] = $this->session->userdata('user_id');
                        }

                        // check duplicate
                        $where = array('mobile' => $data['mobile'], 'first_name' => $data['first_name'], 'last_name' => $data['last_name']);
                        $check_leads = $this->items_model->check_update('tbl_leads', $where);
                        if (empty($check_leads)) 
                        {
                            // save to tbl_leads
                            $this->items_model->_table_name = 'tbl_leads';
                            $this->items_model->_primary_key = 'leads_id';
                            $this->items_model->save($data);
                        }
                        else
                        {
                            $duplicatecnt++;
                            $duplicate_data = array(
                                'first_name' => $data['first_name'],
                                'last_name' => $data['last_name'],
                                'mobile' => $data['mobile']
                            );
                        }
                    }
                    else
                    {
                        $invalid++;
                    }                    
                }
                $type = 'success';
                $total_rows = count($sheetData)-1;
                $total_imported_rows = $total_rows - ($invalidcnt+$duplicatecnt);
                $message = 'Total no of dataleads = '.$total_rows .'</br>';
                $message .= 'Total invalied dataleads = '.$invalidcnt.'</br>';
                $message .= 'Total duplicate dataleads = '.$duplicatecnt.'</br>';
                $message .= 'Total imported dataleads = '.$total_imported_rows.'</br>';
                //$message = lang('save_leads');
            } else {
                $type = 'error';
                $message = "Sorry your uploaded file type not allowed ! please upload XLS/CSV File ";
            }
        } else {
            $type = 'error';
            $message = "You did not Select File! please upload XLS/CSV File ";
        }
        set_message($type, $message);
        redirect('admin/dataleads/import_data_leads');
    }
}









