<?php

if (!defined('BASEPATH'))
    exit('No direct script access allowed');

class Leads extends Admin_Controller
{

    public function __construct()
    {
        parent::__construct();
        $this->load->model('items_model');
        $this->load->model('client_model');
        $this->load->model('quotations_model');        
        $this->load->library('form_validation');
        $this->load->helper('ckeditor');
        $this->data['ckeditor'] = array(
            'id' => 'ck_editor',
            'path' => 'asset/js/ckeditor',
            'config' => array(
                'toolbar' => "Full",
                'width' => "99.8%",
                'height' => "400px"
            )
        );
    }

    public function index($id = NULL)
    {
        $data['title'] = lang('all_leads');
        $data['dropzone'] = true;
        if (!empty($id)) {
            if ($id == 'kanban') {
                $data['active'] = 1;
                $k_session['leads_kanban'] = $id;
                $this->session->set_userdata($k_session);
            } elseif ($id == 'list') {
                $data['active'] = 1;
                $this->session->unset_userdata('leads_kanban');
            } elseif ($id == 'by_status') {
                $data['active'] = 1;
                $lead_status_id = $this->uri->segment(5);
                if ($lead_status_id == 'all') {
                    redirect('admin/leads');
                } else {
                    $search = true;
                    $by_status = $this->items_model->get_leads('by_status', $lead_status_id);
                }
                $this->session->unset_userdata('leads_kanban');
            } elseif ($id == 'by_source') {
                $data['active'] = 1;
                $lead_source_id = $this->uri->segment(5);
                if ($lead_source_id == 'all') {
                    redirect('admin/leads');
                } else {
                    $search = true;
                    $by_status = $this->items_model->get_leads('by_source', $lead_source_id);
                }
                $this->session->unset_userdata('leads_kanban');
            } else {
                $data['active'] = 2;
                
                //$can_edit = $this->items_model->can_action('tbl_leads', 'edit', array('leads_id' => $id));
                $edited = can_action('55', 'edited');
                if (!empty($edited)/*!empty($can_edit)*/) {
                    $data['leads_info'] = $this->items_model->check_by(array('leads_id' => $id), 'tbl_leads');
                    $data['districts'] = $this->items_model->getAllDistrictsByStateId($data['leads_info']->state_id);
                    $data['subcategories'] = $this->items_model->getAllSubcategoriesByCatId($data['leads_info']->lead_category_id);
                }
                $this->session->unset_userdata('leads_kanban');
            }

        } else {
            
            if($this->session->userdata('last_fun')=='assigned')
            {
                $data['active'] = 3;
                unset($_SESSION["last_fun"]);
            }
            else
            {
                $data['active'] = 1;
            }
            
        }
        
        // get all leads status
        $status_info = $this->db->order_by('order_no', 'ASC')->get('tbl_lead_status')->result();
        if (!empty($status_info)) {
            foreach ($status_info as $v_status) {
                $data['status_info'][$v_status->lead_type][] = $v_status;
            }
        }
        $data['assign_user'] = $this->items_model->allowed_user('55');
        if (!empty($search)) {
            $data['all_leads'] = $by_status;
        } else {
            $data['all_leads'] = $this->items_model->get_permission('tbl_leads');
        }

        if($this->session->userdata('designations')=='Marketing Head')
        {
            $data['salesmanagers'] = $this->items_model->get_all_salesmanagers();
        }

        $data['states'] = $this->items_model->get_all_states();
        $data['categories'] = $this->items_model->get_all_lead_categories();
        $data['products'] = $this->items_model->get_all_products();
        $data['supplier'] = $this->items_model->get_manufacturer_list('Electricity');
        $data['subview'] = $this->load->view('admin/leads/all_leads', $data, TRUE);              
        $this->load->view('admin/_layout_main', $data); //page load
    }

    public function leadList($filterBy = null, $search_by = null)
    {
        if ($this->input->is_ajax_request()) {
            $this->load->model('datatables');
            $this->datatables->table = 'tbl_leads';
            $custom_field = custom_form_table_search(5);
            $main_column = array('first_name', 'middle_name', 'last_name', 'mobile', 'city', 'address','lead_status');
            $action_array = array('leads_id');
            $result = array_merge($main_column, $custom_field, $action_array);
            $this->datatables->column_order = $result;
            $this->datatables->column_search = $result;
            $this->datatables->order = array('leads_id' => 'ASC');

            $where = array();
            if (!empty($search_by)) {
                if ($search_by == 'by_status') {
                    $where = array('lead_status_id' => $filterBy);
                }
                if ($search_by == 'by_source') {
                    $where = array('lead_source_id' => $filterBy);
                }

            } else {
                if ($filterBy == 'assigned_to_me') {
                    $user_id = $this->session->userdata('user_id');
                    $where = $user_id;
                }
                if ($filterBy == 'everyone') {
                    $where = array('permission' => 'all');
                }
            }

            if($this->session->userdata('designations')=='Accountant')
            {
                $where = array('lead_status' => 'Agreement Generated');
            }

            if($this->session->userdata('designations')=='Quotation Maker')
            {
                $where = array('lead_status' => 'Assigned');
            }

            if($this->session->userdata('designations')=='Sales Manager')
            {
                $where = array('assigned_to' => $this->session->userdata('user_id'));
            }

            if($this->session->userdata('designations')=='Customer Service Executive')
            {
                $where = array('call_operator_id' => $this->session->userdata('user_id'));
            }
            
            if(empty($where))
            {
                $where = array('type' => 'Lead');
            }
            else
            {
                $where = array_merge($where, array('type' => 'Lead'));
            }

            $fetch_data = $this->datatables->get_leads($filterBy, $search_by, $where,$or_where);

            $data = array();

            $edited = can_action('55', 'edited');
            $deleted = can_action('55', 'deleted');
            foreach ($fetch_data as $_key => $v_leads) {
                $action = null;
                if (!empty($v_leads) && $v_leads->converted_client_id == 0) {
                    $can_edit = $this->items_model->can_action('tbl_leads', 'edit', array('leads_id' => $v_leads->leads_id));
                    $can_delete = $this->items_model->can_action('tbl_leads', 'delete', array('leads_id' => $v_leads->leads_id));

                    $sub_array = array();
                    if (!empty($deleted)) {
                        $sub_array[] = '<div class="checkbox c-checkbox" ><label class="needsclick"> <input name="ids[]" value="' . $v_leads->leads_id . '" type="checkbox"><span class="fa fa-check"></span></label></div>';
                    }
                    $name = null;
                    $name .= '<a class="text-info" href="' . base_url() . 'admin/leads/leads_details/' . $v_leads->leads_id . '">' . $v_leads->first_name . ' ' .$v_leads->middle_name . ' ' . $v_leads->last_name . '</a>';
                    //$name .= $v_leads->first_name . ' ' .$v_leads->middle_name . ' ' . $v_leads->last_name;
                    $sub_array[] = $name;
                    $sub_array[] = $v_leads->mobile;
                    $sub_array[] = $v_leads->city;
                    $sub_array[] = $v_leads->address;
                    $sub_array[] = $v_leads->lead_status;

                    if($this->session->userdata('designations')=='Marketing Head')
                    {
                        if(!empty($v_leads->assigned_to))
                        {
                            // get salesmanagers name
                            $salesmanager = $this->items_model->check_by(array('user_id' => $v_leads->assigned_to),'tbl_account_details');
                            $sub_array[] = $salesmanager->fullname;
                        }
                        else
                        {
                            $sub_array[] = '-';
                        }
                    }

                    if($this->session->userdata('designations')!='Customer Service Executive' )
                    { 
                        
                        if (!empty($edited)) {
                            if($this->session->userdata('designations')=='Sales Manager')
                            {
                                $quotation_list = $this->quotations_model->getQuotationsById($v_leads->leads_id);
                                if($v_leads->is_appointment == 'Yes' && ($v_leads->lead_status =='Assigned'))
                                {
                                    $action .= btn_edit('admin/leads/index/' . $v_leads->leads_id) . ' ';
                                }
                                if(!empty($quotation_list))
                                {
                                    $action .= btn_view('admin/leads/quotationlist/' . $v_leads->leads_id) . ' ';
                                }
                            }
                            else
                            {
                                $action .= btn_edit('admin/leads/index/' . $v_leads->leads_id) . ' ';
                            }
                            
                        }
                        if (!empty($deleted)) {
                            $action .= ajax_anchor(base_url("admin/leads/delete_leads/$v_leads->leads_id"), "<i class='btn btn-xs btn-danger fa fa-trash-o'></i>", array("class" => "", "title" => lang('delete'), "data-fade-out-on-success" => "#table_" . $_key)) . ' ';
                        }                
                        if($this->session->userdata('designations') == 'Accountant')         
                        {
                            $action .="<a href=" .  base_url(). "admin/leads/convert_to_customer/". $v_leads->leads_id ." class='btn-xs btn btn-purple '><i class='fa fa-copy'></i>". lang('convert_to_project') ."</a>";
                        }

                    }
                    $sub_array[] = $action;
                    $data[] = $sub_array;
                }

            }
            render_table($data, $where, null, $or_where);
        } else {
            redirect('admin/dashboard');
        }
    }
    
    public function lead_status()
    {
        $data['title'] = lang('lead_status');
        $data['subview'] = $this->load->view('admin/leads/lead_status', $data, FALSE);
        $this->load->view('admin/_layout_modal', $data);
    }

    public function update_lead_status($id = null)
    {
        $this->items_model->_table_name = 'tbl_lead_status';
        $this->items_model->_primary_key = 'lead_status_id';

        $cate_data['lead_status'] = $this->input->post('lead_status', TRUE);
        $cate_data['lead_type'] = $this->input->post('lead_type', TRUE);
        $cate_data['order_no'] = $this->input->post('order_no', TRUE);

        // update root category
        $where = array('lead_status' => $cate_data['lead_status']);
        // duplicate value check in DB
        if (!empty($id)) { // if id exist in db update data
            $lead_status_id = array('lead_status_id !=' => $id);
        } else { // if id is not exist then set id as null
            $lead_status_id = null;
        }
        // check whether this input data already exist or not
        $check_lead_status = $this->items_model->check_update('tbl_lead_status', $where, $lead_status_id);
        if (!empty($check_lead_status)) { // if input data already exist show error alert
            // massage for user
            $type = 'error';
            $msg = "<strong style='color:#000'>" . $cate_data['lead_status'] . '</strong>  ' . lang('already_exist');
        } else { // save and update query
            $id = $this->items_model->save($cate_data, $id);

            $activity = array(
                'user' => $this->session->userdata('user_id'),
                'module' => 'settings',
                'module_field_id' => $id,
                'activity' => ('activity_added_a_lead_status'),
                'value1' => $cate_data['lead_status']
            );
            $this->items_model->_table_name = 'tbl_activities';
            $this->items_model->_primary_key = 'activities_id';
            $this->items_model->save($activity);

            // messages for user
            $type = "success";
            $msg = lang('lead_status_added');
        }
        if (!empty($id)) {
            $result = array(
                'id' => $id,
                'lead_status' => $cate_data['lead_status'],
                'status' => $type,
                'message' => $msg,
            );
        } else {
            $result = array(
                'status' => $type,
                'message' => $msg,
            );
        }
        echo json_encode($result);
        exit();
    }

    public function lead_source()
    {
        $data['title'] = lang('lead_source');
        $data['subview'] = $this->load->view('admin/leads/lead_source', $data, FALSE);
        $this->load->view('admin/_layout_modal', $data);
    }

    public function update_lead_source($id = null)
    {
        $this->items_model->_table_name = 'tbl_lead_source';
        $this->items_model->_primary_key = 'lead_source_id';

        $source_data['lead_source'] = $this->input->post('lead_source', TRUE);
        // update root category
        $where = array('lead_source' => $source_data['lead_source']);
        // duplicate value check in DB
        if (!empty($id)) { // if id exist in db update data
            $lead_source_id = array('lead_source_id !=' => $id);
        } else { // if id is not exist then set id as null
            $lead_source_id = null;
        }
        // check whether this input data already exist or not
        $check_lead_status = $this->items_model->check_update('tbl_lead_source', $where, $lead_source_id);
        if (!empty($check_lead_status)) { // if input data already exist show error alert
            // massage for user
            $type = 'error';
            $msg = "<strong style='color:#000'>" . $source_data['lead_source'] . '</strong>  ' . lang('already_exist');
        } else { // save and update query
            $id = $this->items_model->save($source_data, $id);

            $activity = array(
                'user' => $this->session->userdata('user_id'),
                'module' => 'settings',
                'module_field_id' => $id,
                'activity' => ('activity_added_a_lead_source'),
                'value1' => $source_data['lead_source']
            );
            $this->items_model->_table_name = 'tbl_activities';
            $this->items_model->_primary_key = 'activities_id';
            $this->items_model->save($activity);

            // messages for user
            $type = "success";
            $msg = lang('lead_source_added');
        }
        if (!empty($id)) {
            $result = array(
                'id' => $id,
                'lead_source' => $source_data['lead_source'],
                'status' => $type,
                'message' => $msg,
            );
        } else {
            $result = array(
                'status' => $type,
                'message' => $msg,
            );
        }
        echo json_encode($result);
        exit();
    }

    public function import_leads()
    {
        $data['title'] = lang('import_leads');
        $data['assign_user'] = $this->items_model->allowed_user('55');
        // get all leads status
        $status_info = $this->db->get('tbl_lead_status')->result();
        if (!empty($status_info)) {
            foreach ($status_info as $v_status) {
                $data['status_info'][$v_status->lead_type][] = $v_status;
            }
        }

        $data['subview'] = $this->load->view('admin/leads/import_leads', $data, TRUE);
        $this->load->view('admin/_layout_main', $data); //page load
    }

    public function save_imported()
    {
        //load the excel library
        $this->load->library('excel');
        ob_start();
        $file = $_FILES["upload_file"]["tmp_name"];
        if (!empty($file)) {
            $valid = false;
            $types = array('Excel2007', 'Excel5','CSV');
            foreach ($types as $type) {
                $reader = PHPExcel_IOFactory::createReader($type);
                if ($reader->canRead($file)) {
                    $valid = true;
                }
            }
            if (!empty($valid)) {
                try {
                    $objPHPExcel = PHPExcel_IOFactory::load($file);
                } catch (Exception $e) {
                    die("Error loading file :" . $e->getMessage());
                }
                //All data from excel
                $sheetData = $objPHPExcel->getActiveSheet()->toArray(null, true, true, true);

                for ($x = 2; $x <= count($sheetData); $x++) {
                    // **********************
                    // Save Into leads table
                    // **********************
                    $data = $this->items_model->array_from_post(array('client_id', 'lead_status_id', 'lead_source_id'));
                    $data['lead_name'] = trim($sheetData[$x]["A"]);
                    $data['organization'] = trim($sheetData[$x]["B"]);
                    $data['contact_name'] = trim($sheetData[$x]["C"]);
                    $data['email'] = trim($sheetData[$x]["D"]);
                    $data['phone'] = trim($sheetData[$x]["E"]);
                    $data['mobile'] = trim($sheetData[$x]["F"]);
                    $data['address'] = trim($sheetData[$x]["G"]);
                    $data['city'] = trim($sheetData[$x]["H"]);
                    $data['country'] = trim($sheetData[$x]["I"]);
                    $data['facebook'] = trim($sheetData[$x]["J"]);
                    $data['skype'] = trim($sheetData[$x]["K"]);
                    $data['twitter'] = trim($sheetData[$x]["L"]);
                    $data['notes'] = trim($sheetData[$x]["M"]);

                    $permission = $this->input->post('permission', true);
                    if (!empty($permission)) {
                        if ($permission == 'everyone') {
                            $assigned = 'all';
                        } else {
                            $assigned_to = $this->items_model->array_from_post(array('assigned_to'));
                            if (!empty($assigned_to['assigned_to'])) {
                                foreach ($assigned_to['assigned_to'] as $assign_user) {
                                    $assigned[$assign_user] = $this->input->post('action_' . $assign_user, true);
                                }
                            }
                        }
                        if (!empty($assigned)) {
                            if ($assigned != 'all') {
                                $assigned = json_encode($assigned);
                            }
                        } else {
                            $assigned = 'all';
                        }
                    } else {
                        $assigned = 'all';
                    }
                    $data['permission'] = $assigned;
                    // save to tbl_leads
                    $this->items_model->_table_name = 'tbl_leads';
                    $this->items_model->_primary_key = 'leads_id';
                    $this->items_model->save($data);
                }
                $type = 'success';
                $message = lang('save_leads');
            } else {
                $type = 'error';
                $message = "Sorry your uploaded file type not allowed ! please upload XLS/CSV File ";
            }
        } else {
            $type = 'error';
            $message = "You did not Select File! please upload XLS/CSV File ";
        }
        set_message($type, $message);
        redirect('admin/leads');
    }

    public function saved_leads($id = NULL) 
    {  
        //echo '<pre>';print_r($_POST);die;
        $created = can_action('55', 'created');
        $edited = can_action('55', 'edited');
        if (!empty($created) || !empty($edited) && !empty($id)) {
            $this->items_model->_table_name = 'tbl_leads';
            $this->items_model->_primary_key = 'leads_id';

            $data = $this->items_model->array_from_post(array('first_name', 'middle_name', 'last_name', 'email', 'mobile', 'address', 'state_id', 'district_id', 'city', 'pincode', 'lead_category_id', 'lead_subcategory_id', 'product_id', 'calculation_type', 'consumer_number', 'supplier', 'units', 'approx_unit_rate', 'unit_type', 'no_of_months', 'lead_source_id', 'lead_type', 'reference','billing_unit','salesmanager_comment'));
            if($this->session->userdata('designations')=='Marketing Head')
            {
                $data['lead_status'] = 'Ready To Assign';
            }
            else if($this->session->userdata('designations') == 'Sales Manager')
            {
                if($this->input->post('save') == 'add')
                {
                    $data['lead_status'] = 'Assigned';
                    $data['assigned_to'] = $this->session->userdata('user_id');
                }
                else if($this->input->post('save') == 'update_and_req_quotation')
                {
                    $data['lead_status'] = 'Request For Quotation';
                }
            }
            else
            {
                $data['lead_status'] = 'New';
            }
            $data['created_by'] = $this->session->userdata('user_id');
            if($this->input->post('lead_source') == 'Own')
            {
                $data['referred_by'] = $this->session->userdata('user_id');
            }
            // check whether this input data already exist or not
            $where = array('mobile' => $data['mobile'], 'first_name' => $data['first_name'], 'last_name' => $data['last_name']);
            $check_leads = $this->items_model->check_duplicate_lead($where, $id);
            if (!empty($check_leads)) { // if input data already exist show error alert
                // massage for user
                $type = 'error';
                $msg = "<strong style='color:#000'>" . $data['first_name'] . ' ' . $data['middle_name'] . ' ' . $data['last_name'] . '</strong>  ' . lang('already_exist');
            } else { // save and update query
                if(!empty($id))
                {
                    $lead = $this->items_model->check_by(array('leads_id' => $id), 'tbl_leads');
                    if($this->session->userdata('designations')!='Sales Manager')
                    {
                        if($lead->calculation_type!=$this->input->post('calculation_type'))
                        {
                            if($this->input->post('calculation_type') == 'unit')
                            {
                                if(is_file($lead->attachment))
                                {
                                    unlink($lead->attachment);
                                }
                                $data['attachment'] = '';
                            }
                            else if($this->input->post('calculation_type') == 'fileupload')
                            {
                                $data['consumer_number'] = '';
                                $data['supplier'] = '';
                                $data['units'] = '';
                                $data['approx_unit_rate'] = '';
                                $data['unit_type'] = '';
                                $data['no_of_months'] = '';
                            }
                        }
                    }
                }

                if($this->input->post('calculation_type') == 'fileupload' || $this->session->userdata('designations')== 'Sales Manager')
                {
                    if (isset($_FILES['attachment']) && $_FILES['attachment']['name'] != '') 
                    {
                        if($id && $lead->calculation_type == 'fileupload')
                        {
                            if(is_file($lead->attachment))
                            {
                                unlink($lead->attachment);
                            }
                        }

                        $file_path = './uploads/lead_attachments/';
                        if (!is_dir($file_path)) 
                        {
                            mkdir($file_path, 0777, TRUE);
                        }
                        $date = date('Y-m-d H:i:s');
                        $name = strtotime($date);
                        $file_ext = explode(".", $_FILES['attachment']['name']);
                        //$ext = pathinfo(basename( $_FILES['attachment']['name']), PATHINFO_EXTENSION);
                        $ext = $file_ext[1];
                        $act_file_path = $file_path . $name.'.'.$ext;
                        move_uploaded_file($_FILES['attachment']['tmp_name'], $act_file_path);
                        $data['attachment'] = $act_file_path;
                    }
                }
                
                $return_id = $this->items_model->save($data, $id);

                if($return_id && (!empty($this->input->post('state_id'))) && (!empty($this->input->post('district_id'))))
                {
                    $year= date("Y");
                    $statePadded = sprintf("%02d", $this->input->post('state_id'));
                    $districtPadded = sprintf("%04d",$this->input->post('district_id'));
                    $idPadded = sprintf("%06d", $return_id);
                    $code=$year.$statePadded.$districtPadded.$idPadded;
                    $this->db->where('leads_id',$return_id);
                    $this->db->update('tbl_leads',array('lead_number' => $code));
                }
                

                if (!empty($id)) {
                    $id = $id;
                    $action = 'Lead updated';
                    $description = 'not_update_leads';
                    $msg = lang('update_leads');
                } else {
                    $id = $return_id;
                    $action = 'Lead added';
                    $description = 'not_save_leads';
                    $msg = lang('save_leads');
                }
                $u_data['index_no'] = $id;
                $id = $this->items_model->save($u_data, $id);

                save_custom_field(5, $id);
                $activity = array(
                    'user' => $this->session->userdata('user_id'),
                    'module' => 'leads',
                    'module_field_id' => $id,
                    'activity' => $action,
                    'icon' => 'fa-rocket',
                    'link' => 'admin/leads/leads_details/' . $id,
                    'value1' => $data['first_name'] . ' ' . $data['middle_name'] . ' ' . $data['last_name']
                );
                $this->items_model->_table_name = 'tbl_activities';
                $this->items_model->_primary_key = 'activities_id';
                $this->items_model->save($activity);
                // messages for user
                $type = "success";

                $leads_info = $this->items_model->check_by(array('leads_id' => $id), 'tbl_leads');                
            }
            $message = $msg;
            set_message($type, $message);
        }
        redirect('admin/leads');
    }
    
    public
    function leads_details($id, $active = NULL, $op_id = NULL)
    {
        $data['title'] = lang('leads_details');
        //get all task information
        $data['leads_details'] = $this->items_model->check_by(array('leads_id' => $id), 'tbl_leads');
        $data['appointment_info'] = $this->items_model->getappointment($id);

        if ($active == 2) {
            $data['active'] = 2;
            $data['sub_active'] = 1;
            $data['sub_metting'] = 1;
        } elseif ($active == 3) {
            $data['active'] = 3;
            $data['sub_active'] = 1;
            $data['sub_metting'] = 1;
        } elseif ($active == 4) {
            $data['active'] = 4;
            $data['sub_active'] = 1;
            $data['sub_metting'] = 1;
        } elseif ($active == 5) {
            $data['active'] = 5;
            $data['sub_active'] = 1;
            $data['sub_metting'] = 1;
        } elseif ($active == 'metting') {
            $data['active'] = 3;
            $data['sub_active'] = 1;
            $data['sub_metting'] = 2;
            $data['mettings_info'] = $this->items_model->check_by(array('mettings_id' => $op_id), 'tbl_mettings');
        } elseif ($active == 'call') {
            $data['active'] = 2;
            $data['sub_active'] = 2;
            $data['call_info'] = $this->items_model->check_by(array('calls_id' => $op_id), 'tbl_calls');
            $data['sub_metting'] = 1;
        } else {
            $data['active'] = 1;
            $data['sub_active'] = 1;
            $data['sub_metting'] = 1;
        }

        $data['subview'] = $this->load->view('admin/leads/leads_details', $data, TRUE);
        $this->load->view('admin/_layout_main', $data);
    }

    public
    function convert($id)
    {
        $data['title'] = lang('convert_to_client'); //Page title
        $data['person'] = 1;

        $data['leads_info'] = $this->items_model->check_by(array('leads_id' => $id), 'tbl_leads');
        $data['modal_subview'] = $this->load->view('admin/leads/_modal_convert', $data, FALSE);
        $this->load->view('admin/_layout_modal', $data);
    }

    public
    function converted($leads_id)
    {
        $data = $this->items_model->array_from_post(array('name', 'email', 'short_note', 'website', 'phone', 'mobile', 'fax', 'address', 'city', 'zipcode', 'currency',
            'skype_id', 'linkedin', 'facebook', 'twitter', 'language', 'country', 'vat', 'hosting_company', 'hostname', 'port', 'password', 'username', 'client_status', 'latitude', 'longitude', 'customer_group_id'));
        if (!empty($_FILES['profile_photo']['name'])) {
            $val = $this->items_model->uploadImage('profile_photo');
            $val == TRUE || redirect('admin/client/manage_client');
            $data['profile_photo'] = $val['path'];
        }
        $data['leads_id'] = $leads_id;
        $data['client_status'] = '0';

        $this->items_model->_table_name = 'tbl_client';
        $this->items_model->_primary_key = "client_id";
        $return_id = $this->items_model->save($data);
        // update to tbl_leads
        $u_data['converted_client_id'] = $return_id;
        $this->items_model->_table_name = 'tbl_leads';
        $this->items_model->_primary_key = "leads_id";
        $this->items_model->save($u_data, $leads_id);
        $action = ('activity_convert_to_client');
        $activities = array(
            'user' => $this->session->userdata('user_id'),
            'module' => 'leads',
            'module_field_id' => $return_id,
            'activity' => $action,
            'icon' => 'fa-rocket',
            'link' => 'admin/leads/leads_details/' . $leads_id,
            'value1' => $data['name']
        );
        $this->items_model->_table_name = 'tbl_activities';
        $this->items_model->_primary_key = "activities_id";
        $this->items_model->save($activities);
        // messages for user
        $type = "success";
        $message = lang('convert_to_client_suucess');
        set_message($type, $message);

        $leads_info = $this->items_model->check_by(array('leads_id' => $leads_id), 'tbl_leads');
        $client_info = $this->items_model->check_by(array('client_id' => $return_id), 'tbl_client');
        $notifiedUsers = array();
        if (!empty($leads_info->permission) && $leads_info->permission != 'all') {
            $permissionUsers = json_decode($leads_info->permission);
            foreach ($permissionUsers as $user => $v_permission) {
                array_push($notifiedUsers, $user);
            }
        } else {
            $notifiedUsers = $this->items_model->allowed_user_id('55');
        }
        if (!empty($notifiedUsers)) {
            foreach ($notifiedUsers as $users) {
                if ($users != $this->session->userdata('user_id')) {
                    add_notification(array(
                        'to_user_id' => $users,
                        'from_user_id' => true,
                        'description' => 'not_lead_converted_to_client',
                        'link' => 'admin/leads/leads_details/' . $leads_info->leads_id,
                        'value' => lang('lead') . ' ' . $leads_info->lead_name . ' ' . lang('client') . ' ' . $client_info->name,
                    ));
                }
            }
            show_notification($notifiedUsers);
        }

        redirect('admin/client/client_details/' . $return_id);
    }

    public
    function update_users($id)
    {
        // get all assign_user
        $can_edit = $this->items_model->can_action('tbl_leads', 'edit', array('leads_id' => $id));
        if (!empty($can_edit)) {
            // get permission user by menu id
            $data['assign_user'] = $this->items_model->allowed_user('55');

            $data['leads_info'] = $this->items_model->check_by(array('leads_id' => $id), 'tbl_leads');
            $data['modal_subview'] = $this->load->view('admin/leads/_modal_users', $data, FALSE);
            $this->load->view('admin/_layout_modal', $data);
        } else {
            set_message('error', lang('there_in_no_value'));
            if (empty($_SERVER['HTTP_REFERER'])) {
                redirect('admin/leads');
            } else {
                redirect($_SERVER['HTTP_REFERER']);
            }
        }
    }

    public
    function update_member($id)
    {
        $can_edit = $this->items_model->can_action('tbl_leads', 'edit', array('leads_id' => $id));
        if (!empty($can_edit)) {
            $leads_info = $this->items_model->check_by(array('leads_id' => $id), 'tbl_leads');

            $permission = $this->input->post('permission', true);
            if (!empty($permission)) {

                if ($permission == 'everyone') {
                    $assigned = 'all';
                } else {
                    $assigned_to = $this->items_model->array_from_post(array('assigned_to'));
                    if (!empty($assigned_to['assigned_to'])) {
                        foreach ($assigned_to['assigned_to'] as $assign_user) {
                            $assigned[$assign_user] = $this->input->post('action_' . $assign_user, true);
                        }
                    }
                }
                if (!empty($assigned)) {
                    if ($assigned != 'all') {
                        $assigned = json_encode($assigned);
                    }
                } else {
                    $assigned = 'all';
                }
                $data['permission'] = $assigned;
            } else {
                set_message('error', lang('assigned_to') . ' Field is required');
                if (empty($_SERVER['HTTP_REFERER'])) {
                    redirect('admin/leads');
                } else {
                    redirect($_SERVER['HTTP_REFERER']);
                }
            }

//save data into table.
            $this->items_model->_table_name = "tbl_leads"; // table name
            $this->items_model->_primary_key = "leads_id"; // $id
            $this->items_model->save($data, $id);

            $msg = lang('update_leads');
            $activity = 'activity_update_leads';

// save into activities
            $activities = array(
                'user' => $this->session->userdata('user_id'),
                'module' => 'leads',
                'module_field_id' => $id,
                'activity' => $activity,
                'icon' => 'fa-rocket',
                'link' => 'admin/leads/leads_details/' . $id,
                'value1' => $leads_info->lead_name,
            );
// Update into tbl_project
            $this->items_model->_table_name = "tbl_activities"; //table name
            $this->items_model->_primary_key = "activities_id";
            $this->items_model->save($activities);

            $notifiedUsers = array();
            if (!empty($leads_info->permission) && $leads_info->permission != 'all') {
                $permissionUsers = json_decode($leads_info->permission);
                foreach ($permissionUsers as $user => $v_permission) {
                    array_push($notifiedUsers, $user);
                }
            } else {
                $notifiedUsers = $this->items_model->allowed_user_id('55');
            }
            if (!empty($notifiedUsers)) {
                foreach ($notifiedUsers as $users) {
                    if ($users != $this->session->userdata('user_id')) {
                        add_notification(array(
                            'to_user_id' => $users,
                            'from_user_id' => true,
                            'description' => 'assign_to_you_the_lead',
                            'link' => 'admin/leads/leads_details/' . $leads_info->leads_id,
                            'value' => lang('lead') . ' ' . $leads_info->lead_name,
                        ));
                    }
                }
                show_notification($notifiedUsers);
            }

            $type = "success";
            $message = $msg;
            set_message($type, $message);

        } else {
            set_message('error', lang('there_in_no_value'));

        }
        if (empty($_SERVER['HTTP_REFERER'])) {
            redirect('admin/leads');
        } else {
            redirect($_SERVER['HTTP_REFERER']);
        }
    }

    public
    function change_status($leads_id, $lead_status_id)
    {
        $can_edit = $this->items_model->can_action('tbl_leads', 'edit', array('leads_id' => $leads_id));
        if (!empty($can_edit)) {
            $data['lead_status_id'] = $lead_status_id;
            $this->items_model->_table_name = 'tbl_leads';
            $this->items_model->_primary_key = 'leads_id';
            $this->items_model->save($data, $leads_id);

            $leads_info = $this->items_model->check_by(array('leads_id' => $leads_id), 'tbl_leads');
            $notifiedUsers = array();
            if (!empty($leads_info->permission) && $leads_info->permission != 'all') {
                $permissionUsers = json_decode($leads_info->permission);
                foreach ($permissionUsers as $user => $v_permission) {
                    array_push($notifiedUsers, $user);
                }
            } else {
                $notifiedUsers = $this->items_model->allowed_user_id('55');
            }
            if (!empty($notifiedUsers)) {
                foreach ($notifiedUsers as $users) {
                    if ($users != $this->session->userdata('user_id')) {
                        add_notification(array(
                            'to_user_id' => $users,
                            'from_user_id' => true,
                            'description' => 'not_changed_status',
                            'link' => 'admin/leads/leads_details/' . $leads_info->leads_id,
                            'value' => lang('lead') . ' ' . $leads_info->lead_name,
                        ));
                    }
                }
                show_notification($notifiedUsers);
            }
            // messages for user
            $type = "success";
            $message = lang('change_status');
            set_message($type, $message);
        } else {
            set_message('error', lang('there_in_no_value'));
        }
        if (empty($_SERVER['HTTP_REFERER'])) {
            redirect('admin/leads');
        } else {
            redirect($_SERVER['HTTP_REFERER']);
        }
    }

    public
    function saved_call($leads_id, $id = NULL)
    {
        $data = $this->items_model->array_from_post(array('date', 'call_summary', 'client_id', 'user_id'));
        $data['leads_id'] = $leads_id;
        $this->items_model->_table_name = 'tbl_calls';
        $this->items_model->_primary_key = 'calls_id';
        $return_id = $this->items_model->save($data, $id);
        if (!empty($id)) {
            $id = $id;
            $action = 'activity_update_leads_call';
            $msg = lang('update_leads_call');
        } else {
            $id = $return_id;
            $action = 'activity_save_leads_call';
            $msg = lang('save_leads_call');
        }
        $activity = array(
            'user' => $this->session->userdata('user_id'),
            'module' => 'leads',
            'module_field_id' => $leads_id,
            'activity' => $action,
            'icon' => 'fa-rocket',
            'link' => 'admin/leads/leads_details/' . $leads_id . '/2',
            'value1' => $data['call_summary']
        );
        $this->items_model->_table_name = 'tbl_activities';
        $this->items_model->_primary_key = 'activities_id';
        $this->items_model->save($activity);

        $leads_info = $this->items_model->check_by(array('leads_id' => $leads_id), 'tbl_leads');
        $notifiedUsers = array();
        if (!empty($leads_info->permission) && $leads_info->permission != 'all') {
            $permissionUsers = json_decode($leads_info->permission);
            foreach ($permissionUsers as $user => $v_permission) {
                array_push($notifiedUsers, $user);
            }
        } else {
            $notifiedUsers = $this->items_model->allowed_user_id('55');
        }
        if (!empty($notifiedUsers)) {
            foreach ($notifiedUsers as $users) {
                if ($users != $this->session->userdata('user_id')) {
                    add_notification(array(
                        'to_user_id' => $users,
                        'from_user_id' => true,
                        'description' => 'not_add_call',
                        'link' => 'admin/leads/leads_details/' . $leads_info->leads_id . '/2',
                        'value' => lang('lead') . ' ' . $leads_info->lead_name,
                    ));
                }
            }
            show_notification($notifiedUsers);
        }
        // messages for user
        $type = "success";
        $message = $msg;
        set_message($type, $message);
        redirect('admin/leads/leads_details/' . $leads_id . '/' . '2');
    }

    public
    function delete_leads_call($leads_id, $id)
    {
        $calls_info = $this->items_model->check_by(array('calls_id' => $id), 'tbl_calls');
        $activity = array(
            'user' => $this->session->userdata('user_id'),
            'module' => 'leads',
            'module_field_id' => $leads_id,
            'activity' => 'activity_leads_call_deleted',
            'icon' => 'fa-rocket',
            'link' => 'admin/leads/leads_details/' . $leads_id . '/2',
            'value1' => $calls_info->call_summary
        );
        $this->items_model->_table_name = 'tbl_activities';
        $this->items_model->_primary_key = 'activities_id';
        $this->items_model->save($activity);

        $this->items_model->_table_name = 'tbl_calls';
        $this->items_model->_primary_key = 'calls_id';
        $this->items_model->delete($id);
        $type = 'success';
        $message = lang('leads_call_deleted');
        // messages for user
        echo json_encode(array("status" => $type, 'message' => $message));
        exit();
    }

    public
    function delete_leads_mettings($leads_id, $id)
    {
        $mettings_info = $this->items_model->check_by(array('mettings_id' => $id), 'tbl_mettings');

        $activity = array(
            'user' => $this->session->userdata('user_id'),
            'module' => 'leads',
            'module_field_id' => $leads_id,
            'activity' => 'activity_leads_call_deleted',
            'icon' => 'fa-rocket',
            'link' => 'admin/leads/leads_details/' . $leads_id . '/3',
            'value1' => $mettings_info->meeting_subject
        );
        $this->items_model->_table_name = 'tbl_activities';
        $this->items_model->_primary_key = 'activities_id';
        $this->items_model->save($activity);
        $this->items_model->_table_name = 'tbl_mettings';
        $this->items_model->_primary_key = 'mettings_id';
        $this->items_model->delete($id);
        $type = 'success';
        $message = lang('leads_mettings_deleted');

        echo json_encode(array("status" => $type, 'message' => $message));
        exit();
    }

    public
    function saved_metting($leads_id, $id = NULL)
    {
        $this->items_model->_table_name = 'tbl_mettings';
        $this->items_model->_primary_key = 'mettings_id';

        $data = $this->items_model->array_from_post(array('meeting_subject', 'user_id', 'location', 'description'));
        $data['start_date'] = strtotime($this->input->post('start_date', true) . ' ' . display_time($this->input->post('start_time', true)));
        $data['end_date'] = strtotime($this->input->post('end_date', true) . ' ' . display_time($this->input->post('end_time', true)));
        $data['leads_id'] = $leads_id;
        $user_id = serialize($this->items_model->array_from_post(array('attendees')));
        if (!empty($user_id)) {
            $data['attendees'] = $user_id;
        } else {
            $data['attendees'] = '-';
        }
        $return_id = $this->items_model->save($data, $id);

        if (!empty($id)) {
            $id = $id;
            $action = 'activity_update_leads_metting';
            $msg = lang('update_leads_metting');
        } else {
            $id = $return_id;
            $action = 'activity_save_leads_metting';
            $msg = lang('save_leads_metting');
        }
        $activity = array(
            'user' => $this->session->userdata('user_id'),
            'module' => 'leads',
            'module_field_id' => $leads_id,
            'activity' => $action,
            'icon' => 'fa-rocket',
            'link' => 'admin/leads/leads_details/' . $leads_id . '/3',
            'value1' => $data['meeting_subject']
        );

        $this->items_model->_table_name = 'tbl_activities';
        $this->items_model->_primary_key = 'activities_id';
        $this->items_model->save($activity);

        $leads_info = $this->items_model->check_by(array('leads_id' => $leads_id), 'tbl_leads');
        $notifiedUsers = array();
        if (!empty($leads_info->permission) && $leads_info->permission != 'all') {
            $permissionUsers = json_decode($leads_info->permission);
            foreach ($permissionUsers as $user => $v_permission) {
                array_push($notifiedUsers, $user);
            }
        } else {
            $notifiedUsers = $this->items_model->allowed_user_id('55');
        }
        if (!empty($notifiedUsers)) {
            foreach ($notifiedUsers as $users) {
                if ($users != $this->session->userdata('user_id')) {
                    add_notification(array(
                        'to_user_id' => $users,
                        'from_user_id' => true,
                        'description' => 'not_add_meetings',
                        'link' => 'admin/leads/leads_details/' . $leads_info->leads_id . '/3',
                        'value' => lang('lead') . ' ' . $leads_info->lead_name,
                    ));
                }
            }
            show_notification($notifiedUsers);
        }
        // messages for user
        $type = "success";
        $message = $msg;
        set_message($type, $message);
        redirect('admin/leads/leads_details/' . $leads_id . '/' . '3');
    }

    public
    function save_comments()
    {
        $data['leads_id'] = $this->input->post('leads_id', TRUE);
        $data['comment'] = $this->input->post('comment', TRUE);

        $files = $this->input->post("files", true);
        $target_path = getcwd() . "/uploads/";
        //process the fiiles which has been uploaded by dropzone
        if (!empty($files) && is_array($files)) {
            foreach ($files as $key => $file) {
                if (!empty($file)) {
                    $file_name = $this->input->post('file_name_' . $file, true);
                    $new_file_name = move_temp_file($file_name, $target_path);
                    $file_ext = explode(".", $new_file_name);
                    $is_image = check_image_extension($new_file_name);
                    $size = $this->input->post('file_size_' . $file, true) / 1000;
                    if ($new_file_name) {
                        $up_data[] = array(
                            "fileName" => $new_file_name,
                            "path" => "uploads/" . $new_file_name,
                            "fullPath" => getcwd() . "/uploads/" . $new_file_name,
                            "ext" => '.' . end($file_ext),
                            "size" => round($size, 2),
                            "is_image" => $is_image,
                        );
                        $success = true;
                    } else {
                        $success = false;
                    }
                }
            }
        }
        //process the files which has been submitted manually
        if ($_FILES) {
            $files = $_FILES['manualFiles'];
            if ($files && count($files) > 0) {
                foreach ($files["tmp_name"] as $key => $file) {
                    $temp_file = $file;
                    $file_name = $files["name"][$key];
                    $file_size = $files["size"][$key];
                    $new_file_name = move_temp_file($file_name, $target_path, "", $temp_file);
                    if ($new_file_name) {
                        $file_ext = explode(".", $new_file_name);
                        $is_image = check_image_extension($new_file_name);
                        $up_data[] = array(
                            "fileName" => $new_file_name,
                            "path" => "uploads/" . $new_file_name,
                            "fullPath" => getcwd() . "/uploads/" . $new_file_name,
                            "ext" => '.' . end($file_ext),
                            "size" => round($file_size, 2),
                            "is_image" => $is_image,
                        );
                    }
                }
            }
        }
        if (!empty($up_data)) {
            $data['comments_attachment'] = json_encode($up_data);
        }

        $data['user_id'] = $this->session->userdata('user_id');

        //save data into table.
        $this->items_model->_table_name = "tbl_task_comment"; // table name
        $this->items_model->_primary_key = "task_comment_id"; // $id
        $comment_id = $this->items_model->save($data);

        // save into activities
        $activities = array(
            'user' => $this->session->userdata('user_id'),
            'module' => 'leads',
            'module_field_id' => $data['leads_id'],
            'activity' => 'activity_new_leads_comment',
            'icon' => 'fa-rocket',
            'link' => 'admin/leads/leads_details/' . $data['leads_id'] . '/4',
            'value1' => $data['comment'],
        );

        // Update into tbl_project
        $this->items_model->_table_name = "tbl_activities"; //table name
        $this->items_model->_primary_key = "activities_id";
        $this->items_model->save($activities);

        if (!empty($comment_id)) {

            $leads_info = $this->items_model->check_by(array('leads_id' => $data['leads_id']), 'tbl_leads');
            $notifiedUsers = array();
            if (!empty($leads_info->permission) && $leads_info->permission != 'all') {
                $permissionUsers = json_decode($leads_info->permission);
                foreach ($permissionUsers as $user => $v_permission) {
                    array_push($notifiedUsers, $user);
                }
            } else {
                $notifiedUsers = $this->items_model->allowed_user_id('55');
            }
            if (!empty($notifiedUsers)) {
                foreach ($notifiedUsers as $users) {
                    if ($users != $this->session->userdata('user_id')) {
                        add_notification(array(
                            'to_user_id' => $users,
                            'from_user_id' => true,
                            'description' => 'not_new_comment',
                            'link' => 'admin/leads/leads_details/' . $leads_info->leads_id . '/4',
                            'value' => lang('lead') . ' ' . $leads_info->lead_name,
                        ));
                    }
                }
                show_notification($notifiedUsers);
            }
            $response_data = "";
            $view_data['comment_details'] = $this->db->where(array('task_comment_id' => $comment_id))->order_by('comment_datetime', 'DESC')->get('tbl_task_comment')->result();
            $response_data = $this->load->view("admin/leads/comments_list", $view_data, true);
            echo json_encode(array("status" => 'success', "data" => $response_data, 'message' => lang('leads_comment_save')));
            exit();
        } else {
            echo json_encode(array("status" => 'error', 'message' => lang('error_occurred')));
            exit();
        }
//        $type = "success";
//        $message = lang('leads_comment_save');
//        set_message($type, $message);
//        redirect('admin/leads/leads_details/' . $data['leads_id'] . '/' . '4');
    }

    public function save_comments_reply($task_comment_id)
    {
        $data['leads_id'] = $this->input->post('leads_id', TRUE);
        $data['comment'] = $this->input->post('reply_comments', TRUE);
        $data['user_id'] = $this->session->userdata('user_id');
        $data['comments_reply_id'] = $task_comment_id;
        //save data into table.
        $this->items_model->_table_name = "tbl_task_comment"; // table name
        $this->items_model->_primary_key = "task_comment_id"; // $id
        $comment_id = $this->items_model->save($data);
        if (!empty($comment_id)) {

            $comments_info = $this->items_model->check_by(array('task_comment_id' => $task_comment_id), 'tbl_task_comment');
            $user = $this->items_model->check_by(array('user_id' => $comments_info->user_id), 'tbl_users');
            if ($user->role_id == 2) {
                $url = 'client/';
            } else {
                $url = 'admin/';
            }
            // save into activities
            $activities = array(
                'user' => $this->session->userdata('user_id'),
                'module' => 'leads',
                'module_field_id' => $data['leads_id'],
                'activity' => 'activity_new_comment_reply',
                'icon' => 'fa-rocket',
                'link' => $url . 'leads/leads_details/' . $data['leads_id'] . '/4',
                'value1' => $this->db->where('task_comment_id', $task_comment_id)->get('tbl_task_comment')->row()->comment,
                'value2' => $data['comment'],
            );
            // Update into tbl_project
            $this->items_model->_table_name = "tbl_activities"; //table name
            $this->items_model->_primary_key = "activities_id";
            $this->items_model->save($activities);

            $leads_info = $this->items_model->check_by(array('leads_id' => $data['leads_id']), 'tbl_leads');

            $notifiedUsers = array($comments_info->user_id);
            if (!empty($notifiedUsers)) {
                foreach ($notifiedUsers as $users) {
                    if ($users != $this->session->userdata('user_id')) {
                        add_notification(array(
                            'to_user_id' => $users,
                            'from_user_id' => true,
                            'description' => 'not_new_comment',
                            'link' => $url . 'leads/leads_details/' . $leads_info->leads_id . '/4',
                            'value' => lang('lead') . ' ' . $leads_info->lead_name,
                        ));
                    }
                }
                show_notification($notifiedUsers);
            }
            $response_data = "";
            $view_data['comment_reply_details'] = $this->db->where(array('task_comment_id' => $comment_id))->order_by('comment_datetime', 'DESC')->get('tbl_task_comment')->result();
            $response_data = $this->load->view("admin/leads/comments_reply", $view_data, true);
            echo json_encode(array("status" => 'success', "data" => $response_data, 'message' => lang('leads_comment_save')));
            exit();
        } else {
            echo json_encode(array("status" => 'error', 'message' => lang('error_occurred')));
            exit();
        }

//        $type = "success";
//        $message = lang('leads_comment_save');
//        set_message($type, $message);
//        redirect('admin/leads/leads_details/' . $data['leads_id'] . '/' . '4');
    }

    public
    function delete_comments($task_comment_id = null)
    {
        $comments_info = $this->items_model->check_by(array('task_comment_id' => $task_comment_id), 'tbl_task_comment');

        if (!empty($comments_info->comments_attachment)) {
            $attachment = json_decode($comments_info->comments_attachment);
            foreach ($attachment as $v_file) {
                remove_files($v_file->fileName);
            }
        }
        // save into activities
        $activities = array(
            'user' => $this->session->userdata('user_id'),
            'module' => 'leads',
            'module_field_id' => $comments_info->leads_id,
            'activity' => 'activity_comment_deleted',
            'icon' => 'fa-rocket',
            'link' => 'admin/leads/leads_details/' . $comments_info->leads_id . '/4',
            'value1' => $comments_info->comment,
        );
        // Update into tbl_project
        $this->items_model->_table_name = "tbl_activities"; //table name
        $this->items_model->_primary_key = "activities_id";
        $this->items_model->save($activities);

        //save data into table.
        $this->items_model->_table_name = "tbl_task_comment"; // table name
        $this->items_model->delete_multiple(array('comments_reply_id' => $task_comment_id));
        //save data into table.
        $this->items_model->_table_name = "tbl_task_comment"; // table name
        $this->items_model->_primary_key = "task_comment_id"; // $id
        $this->items_model->delete($task_comment_id);

        echo json_encode(array("status" => 'success', 'message' => lang('task_comment_deleted')));
        exit();
    }

    public
    function save_attachment($task_attachment_id = NULL)
    {
        $data = $this->items_model->array_from_post(array('title', 'description', 'leads_id'));
        $data['user_id'] = $this->session->userdata('user_id');

        // save and update into tbl_files
        $this->items_model->_table_name = "tbl_task_attachment"; //table name
        $this->items_model->_primary_key = "task_attachment_id";
        if (!empty($task_attachment_id)) {
            $id = $task_attachment_id;
            $this->items_model->save($data, $id);
            $msg = lang('leads_file_updated');
        } else {
            $id = $this->items_model->save($data);
            $msg = lang('leads_file_added');
        }
        $files = $this->input->post("files", true);

        $target_path = getcwd() . "/uploads/";
        //process the fiiles which has been uploaded by dropzone
        if (!empty($files) && is_array($files)) {
            foreach ($files as $key => $file) {
                if (!empty($file)) {
                    $file_name = $this->input->post('file_name_' . $file, true);
                    $new_file_name = move_temp_file($file_name, $target_path);
                    $file_ext = explode(".", $new_file_name);
                    $is_image = check_image_extension($new_file_name);

                    if ($new_file_name) {
                        $up_data = array(
                            "files" => "uploads/" . $new_file_name,
                            "uploaded_path" => getcwd() . "/uploads/" . $new_file_name,
                            "file_name" => $new_file_name,
                            "size" => $this->input->post('file_size_' . $file, true),
                            "ext" => end($file_ext),
                            "is_image" => $is_image,
                            "image_width" => 0,
                            "image_height" => 0,
                            "task_attachment_id" => $id
                        );
                        $this->items_model->_table_name = "tbl_task_uploaded_files"; // table name
                        $this->items_model->_primary_key = "uploaded_files_id"; // $id
                        $uploaded_files_id = $this->items_model->save($up_data);

                        // saved into comments
                        $comment = $this->input->post('comment_' . $file, true);
                        if (!empty($comment)) {
                            $u_cdata = array(
                                "comment" => $comment,
                                "leads_id" => $data['leads_id'],
                                "user_id" => $this->session->userdata('user_id'),
                                "uploaded_files_id" => $uploaded_files_id,
                            );
                            $this->items_model->_table_name = "tbl_task_comment"; // table name
                            $this->items_model->_primary_key = "task_comment_id"; // $id
                            $this->items_model->save($u_cdata);
                        }
                        $success = true;
                    } else {
                        $success = false;
                    }
                }
            }
        }
        //process the files which has been submitted manually
        if ($_FILES) {
            $files = $_FILES['manualFiles'];
            if ($files && count($files) > 0) {
                $comment = $this->input->post('comment', true);
                foreach ($files["tmp_name"] as $key => $file) {
                    $temp_file = $file;
                    $file_name = $files["name"][$key];
                    $file_size = $files["size"][$key];
                    $new_file_name = move_temp_file($file_name, $target_path, "", $temp_file);
                    if ($new_file_name) {
                        $file_ext = explode(".", $new_file_name);
                        $is_image = check_image_extension($new_file_name);
                        $up_data = array(
                            "files" => "uploads/" . $new_file_name,
                            "uploaded_path" => getcwd() . "/uploads/" . $new_file_name,
                            "file_name" => $new_file_name,
                            "size" => $file_size,
                            "ext" => end($file_ext),
                            "is_image" => $is_image,
                            "image_width" => 0,
                            "image_height" => 0,
                            "task_attachment_id" => $id
                        );
                        $this->items_model->_table_name = "tbl_task_uploaded_files"; // table name
                        $this->items_model->_primary_key = "uploaded_files_id"; // $id
                        $uploaded_files_id = $this->items_model->save($up_data);

                        // saved into comments
                        if (!empty($comment[$key])) {
                            $u_cdata = array(
                                "comment" => $comment[$key],
                                "leads_id" => $data['leads_id'],
                                "user_id" => $this->session->userdata('user_id'),
                                "uploaded_files_id" => $uploaded_files_id,
                            );
                            $this->items_model->_table_name = "tbl_task_comment"; // table name
                            $this->items_model->_primary_key = "task_comment_id"; // $id
                            $this->items_model->save($u_cdata);
                        }

                    }
                }
            }
        }
        // save into activities
        $activities = array(
            'user' => $this->session->userdata('user_id'),
            'module' => 'leads',
            'module_field_id' => $data['leads_id'],
            'activity' => 'activity_new_leads_attachment',
            'icon' => 'fa-rocket',
            'link' => 'admin/leads/leads_details/' . $data['leads_id'] . '/5',
            'value1' => $data['title'],
        );
        // Update into tbl_project
        $this->items_model->_table_name = "tbl_activities"; //table name
        $this->items_model->_primary_key = "activities_id";
        $this->items_model->save($activities);

        $leads_info = $this->items_model->check_by(array('leads_id' => $data['leads_id']), 'tbl_leads');
        $notifiedUsers = array();
        if (!empty($leads_info->permission) && $leads_info->permission != 'all') {
            $permissionUsers = json_decode($leads_info->permission);
            foreach ($permissionUsers as $user => $v_permission) {
                array_push($notifiedUsers, $user);
            }
        } else {
            $notifiedUsers = $this->items_model->allowed_user_id('55');
        }
        if (!empty($notifiedUsers)) {
            foreach ($notifiedUsers as $users) {
                if ($users != $this->session->userdata('user_id')) {
                    add_notification(array(
                        'to_user_id' => $users,
                        'from_user_id' => true,
                        'description' => 'not_uploaded_attachment',
                        'link' => 'admin/leads/leads_details/' . $leads_info->leads_id . '/5',
                        'value' => lang('lead') . ' ' . $leads_info->lead_name,
                    ));
                }
            }
            show_notification($notifiedUsers);
        }
        // messages for user
        $type = "success";
        $message = $msg;
        set_message($type, $message);
        redirect('admin/leads/leads_details/' . $data['leads_id'] . '/' . '5');
    }

    public function new_attachment($id)
    {
        $data['dropzone'] = true;
        $data['leads_details'] = $this->items_model->check_by(array('leads_id' => $id), 'tbl_leads');
        $data['modal_subview'] = $this->load->view('admin/leads/new_attachment', $data, FALSE);
        $this->load->view('admin/_layout_modal', $data);
    }

    public function attachment_details($type, $id)
    {
        $data['type'] = $type;
        $data['attachment_info'] = $this->items_model->check_by(array('task_attachment_id' => $id), 'tbl_task_attachment');
        $data['modal_subview'] = $this->load->view('admin/leads/attachment_details', $data, FALSE);
        $this->load->view('admin/_layout_modal_extra_lg', $data);
    }

    public function save_attachment_comments()
    {
        $task_attachment_id = $this->input->post('task_attachment_id', true);
        if (!empty($task_attachment_id)) {
            $data['task_attachment_id'] = $task_attachment_id;
        } else {
            $data['uploaded_files_id'] = $this->input->post('uploaded_files_id', true);
        }
        $data['leads_id'] = $this->input->post('leads_id', true);
        $data['comment'] = $this->input->post('description', true);

        $files = $this->input->post("files", true);
        $target_path = getcwd() . "/uploads/";
        //process the fiiles which has been uploaded by dropzone
        if (!empty($files) && is_array($files)) {
            foreach ($files as $key => $file) {
                if (!empty($file)) {
                    $file_name = $this->input->post('file_name_' . $file, true);
                    $new_file_name = move_temp_file($file_name, $target_path);
                    $file_ext = explode(".", $new_file_name);
                    $is_image = check_image_extension($new_file_name);
                    $size = $this->input->post('file_size_' . $file, true) / 1000;
                    if ($new_file_name) {
                        $up_data[] = array(
                            "fileName" => $new_file_name,
                            "path" => "uploads/" . $new_file_name,
                            "fullPath" => getcwd() . "/uploads/" . $new_file_name,
                            "ext" => '.' . end($file_ext),
                            "size" => round($size, 2),
                            "is_image" => $is_image,
                        );
                        $success = true;
                    } else {
                        $success = false;
                    }
                }
            }
        }
        //process the files which has been submitted manually
        if ($_FILES) {
            $files = $_FILES['manualFiles'];
            if ($files && count($files) > 0) {
                $comment = $this->input->post('comment', true);
                foreach ($files["tmp_name"] as $key => $file) {
                    $temp_file = $file;
                    $file_name = $files["name"][$key];
                    $file_size = $files["size"][$key];
                    $new_file_name = move_temp_file($file_name, $target_path, "", $temp_file);
                    if ($new_file_name) {
                        $file_ext = explode(".", $new_file_name);
                        $is_image = check_image_extension($new_file_name);
                        $up_data[] = array(
                            "fileName" => $new_file_name,
                            "path" => "uploads/" . $new_file_name,
                            "fullPath" => getcwd() . "/uploads/" . $new_file_name,
                            "ext" => '.' . end($file_ext),
                            "size" => round($file_size, 2),
                            "is_image" => $is_image,
                        );
                    }
                }
            }
        }
        if (!empty($up_data)) {
            $data['comments_attachment'] = json_encode($up_data);
        }
        $data['user_id'] = $this->session->userdata('user_id');

        //save data into table.
        $this->items_model->_table_name = "tbl_task_comment"; // table name
        $this->items_model->_primary_key = "task_comment_id"; // $id
        $comment_id = $this->items_model->save($data);
        if (!empty($comment_id)) {
            // save into activities
            $activities = array(
                'user' => $this->session->userdata('user_id'),
                'module' => 'leads',
                'module_field_id' => $data['leads_id'],
                'activity' => 'activity_new_leads_comment',
                'icon' => 'fa-filter',
                'link' => 'admin/leads/leads_details/' . $data['leads_id'] . '/5',
                'value1' => $data['comment'],
            );
            // Update into tbl_project
            $this->items_model->_table_name = "tbl_activities"; //table name
            $this->items_model->_primary_key = "activities_id";
            $this->items_model->save($activities);

            $notifiedUsers = array();
            $leads_info = $this->items_model->check_by(array('leads_id' => $data['leads_id']), 'tbl_leads');
            $notifiedUsers = array();
            if (!empty($leads_info->permission) && $leads_info->permission != 'all') {
                $permissionUsers = json_decode($leads_info->permission);
                foreach ($permissionUsers as $user => $v_permission) {
                    array_push($notifiedUsers, $user);
                }
            } else {
                $notifiedUsers = $this->items_model->allowed_user_id('55');
            }
            if (!empty($notifiedUsers)) {
                foreach ($notifiedUsers as $users) {
                    if ($users != $this->session->userdata('user_id')) {
                        add_notification(array(
                            'to_user_id' => $users,
                            'from_user_id' => true,
                            'description' => 'not_new_comment',
                            'link' => 'admin/leads/leads_details/' . $leads_info->leads_id . '/5',
                            'value' => lang('lead') . ' ' . $leads_info->lead_name,
                        ));
                    }
                }
                show_notification($notifiedUsers);
            }
            $response_data = "";
            $view_data['comment_details'] = $this->db->where(array('task_comment_id' => $comment_id))->order_by('comment_datetime', 'DESC')->get('tbl_task_comment')->result();
            $response_data = $this->load->view("admin/leads/comments_list", $view_data, true);
            echo json_encode(array("status" => 'success', "data" => $response_data, 'message' => lang('leads_comment_save')));
            exit();
        } else {
            echo json_encode(array("status" => 'error', 'message' => lang('error_occurred')));
            exit();
        }

    }


    public
    function delete_files($task_attachment_id)
    {
        $file_info = $this->items_model->check_by(array('task_attachment_id' => $task_attachment_id), 'tbl_task_attachment');
        // save into activities
        $activities = array(
            'user' => $this->session->userdata('user_id'),
            'module' => 'leads',
            'module_field_id' => $file_info->leads_id,
            'activity' => 'activity_leads_attachfile_deleted',
            'icon' => 'fa-rocket',
            'link' => 'admin/leads/leads_details/' . $file_info->leads_id . '/5',
            'value1' => $file_info->title,
        );
        // Update into tbl_project
        $this->items_model->_table_name = "tbl_activities"; //table name
        $this->items_model->_primary_key = "activities_id";
        $this->items_model->save($activities);

        //save data into table.
        $this->items_model->_table_name = "tbl_task_attachment"; // table name
        $this->items_model->delete_multiple(array('task_attachment_id' => $task_attachment_id));


        $uploadFileinfo = $this->db->where('task_attachment_id', $task_attachment_id)->get('tbl_task_uploaded_files')->result();
        if (!empty($uploadFileinfo)) {
            foreach ($uploadFileinfo as $Fileinfo) {
                remove_files($Fileinfo->file_name);
            }
        }
        //save data into table.
        $this->items_model->_table_name = "tbl_task_uploaded_files"; // table name
        $this->items_model->delete_multiple(array('task_attachment_id' => $task_attachment_id));

        echo json_encode(array("status" => 'success', 'message' => lang('leads_attachfile_deleted')));
        exit();
//        $type = "success";
//        $message = lang('leads_attachfile_deleted');
//        set_message($type, $message);
//        redirect('admin/leads/leads_details/' . $leads_id . '/' . '5');
    }

    public function download_files($uploaded_files_id, $comments = null)
    {
        $this->load->helper('download');
        if (!empty($comments)) {
            if ($uploaded_files_id) {
                $down_data = file_get_contents('uploads/' . $uploaded_files_id); // Read the file's contents
                force_download($uploaded_files_id, $down_data);
            } else {
                $type = "error";
                $message = 'Operation Fieled !';
                set_message($type, $message);
                if (empty($_SERVER['HTTP_REFERER'])) {
                    redirect('admin/leads');
                } else {
                    redirect($_SERVER['HTTP_REFERER']);
                }
            }
        } else {
            $uploaded_files_info = $this->items_model->check_by(array('uploaded_files_id' => $uploaded_files_id), 'tbl_task_uploaded_files');
            if ($uploaded_files_info->uploaded_path) {
                $data = file_get_contents($uploaded_files_info->uploaded_path); // Read the file's contents
                force_download($uploaded_files_info->file_name, $data);
            } else {
                $type = "error";
                $message = lang('operation_failed');
                set_message($type, $message);
                if (empty($_SERVER['HTTP_REFERER'])) {
                    redirect('admin/leads');
                } else {
                    redirect($_SERVER['HTTP_REFERER']);
                }
            }
        }
    }

    public
    function download_all_files($attachment_id)
    {
        $uploaded_files_info = $this->db->where('task_attachment_id', $attachment_id)->get('tbl_task_uploaded_files')->result();

        $attachment_info = $this->db->where('task_attachment_id', $attachment_id)->get('tbl_task_attachment')->row();
        $this->load->library('zip');
        if (!empty($uploaded_files_info)) {
            $filename = slug_it($attachment_info->title);
            foreach ($uploaded_files_info as $v_files) {
                $down_data = ($v_files->files); // Read the file's contents
                $this->zip->read_file($down_data);
            }
            $this->zip->download($filename . '.zip');
        } else {
            $type = "error";
            $message = lang('operation_failed');
            set_message($type, $message);
            if (empty($_SERVER['HTTP_REFERER'])) {
                redirect('admin/leads');
            } else {
                redirect($_SERVER['HTTP_REFERER']);
            }
        }
    }

    public function bulk_delete()
    {
        $selected_id = $this->input->post('ids', true);
        if (!empty($selected_id)) {
            foreach ($selected_id as $id) {
                $result[] = $this->delete_leads($id, true);
            }
            echo json_encode($result);
            exit();
        } else {
            $type = "error";
            $message = lang('you_need_select_to_delete');
            echo json_encode(array("status" => $type, 'message' => $message));
            exit();
        }
    }

    public
    function delete_leads($id, $bulk = null)
    {
        $deleted = can_action('55', 'deleted');
        $can_delete = $this->items_model->can_action('tbl_leads', 'delete', array('leads_id' => $id));
        if (!empty($deleted) && !empty($can_delete)) {
            $leads_info = $this->items_model->check_by(array('leads_id' => $id), 'tbl_leads');
            $activity = array(
                'user' => $this->session->userdata('user_id'),
                'module' => 'leads',
                'module_field_id' => $id,
                'activity' => 'activity_leads_deleted',
                'icon' => 'fa-rocket',
                'value1' => $leads_info->lead_name
            );
            $this->items_model->_table_name = 'tbl_activities';
            $this->items_model->_primary_key = 'activities_id';
            $this->items_model->save($activity);

            //delete data into table.
            $this->items_model->_table_name = "tbl_calls"; // table name
            $this->items_model->delete_multiple(array('leads_id' => $id));

            //delete data into table.
            $this->items_model->_table_name = "tbl_mettings"; // table name
            $this->items_model->delete_multiple(array('leads_id' => $id));

            // deleted comments with file
            $all_comments_info = $this->db->where(array('leads_id' => $id))->get('tbl_task_comment')->result();
            if (!empty($all_comments_info)) {
                foreach ($all_comments_info as $comments_info) {
                    if (!empty($comments_info->comments_attachment)) {
                        $attachment = json_decode($comments_info->comments_attachment);
                        foreach ($attachment as $v_file) {
                            remove_files($v_file->fileName);
                        }
                    }
                }
                //delete data into table.
                $this->items_model->_table_name = "tbl_task_comment"; // table name
                $this->items_model->delete_multiple(array('leads_id' => $id));
            }

            $this->items_model->_table_name = "tbl_task_attachment"; //table name
            $this->items_model->_order_by = "leads_id";
            $files_info = $this->items_model->get_by(array('leads_id' => $id), FALSE);
            if (!empty($files_info)) {
                foreach ($files_info as $v_files) {

                    $uploadFileinfo = $this->db->where('task_attachment_id', $v_files->task_attachment_id)->get('tbl_task_uploaded_files')->result();
                    if (!empty($uploadFileinfo)) {
                        foreach ($uploadFileinfo as $Fileinfo) {
                            remove_files($Fileinfo->file_name);
                        }
                    }
                    //save data into table.
                    $this->items_model->_table_name = "tbl_task_uploaded_files"; // table name
                    $this->items_model->delete_multiple(array('task_attachment_id' => $v_files->task_attachment_id));
                }
                //save data into table.
                $this->items_model->_table_name = "tbl_task_attachment"; // table name
                $this->items_model->delete_multiple(array('leads_id' => $id));
            }
            // deleted leads tasks and task comments , attachments,timer
            $leads_tasks = $this->db->where('leads_id', $id)->get('tbl_task')->result();
            if (!empty($leads_tasks)) {
                foreach ($leads_tasks as $v_taks) {

                    $all_comments_info = $this->db->where(array('task_id' => $v_taks->task_id))->get('tbl_task_comment')->result();
                    if (!empty($all_comments_info)) {
                        foreach ($all_comments_info as $comments_info) {
                            if (!empty($comments_info->comments_attachment)) {
                                $attachment = json_decode($comments_info->comments_attachment);
                                foreach ($attachment as $v_file) {
                                    remove_files($v_file->fileName);
                                }
                            }
                        }
                    }
                    //delete data into table.
                    $this->items_model->_table_name = "tbl_task_comment"; // table name
                    $this->items_model->delete_multiple(array('task_id' => $v_taks->task_id));

                    $this->items_model->_table_name = "tbl_task_attachment"; //table name
                    $this->items_model->_order_by = "task_id";
                    $files_info = $this->items_model->get_by(array('task_id' => $v_taks->task_id), FALSE);
                    if (!empty($files_info)) {
                        foreach ($files_info as $t_v_files) {
                            $uploadFileinfo = $this->db->where('task_attachment_id', $t_v_files->task_attachment_id)->get('tbl_task_uploaded_files')->result();
                            if (!empty($uploadFileinfo)) {
                                foreach ($uploadFileinfo as $Fileinfo) {
                                    remove_files($Fileinfo->file_name);
                                }
                            }
                            $this->items_model->_table_name = "tbl_task_uploaded_files"; //table name
                            $this->items_model->delete_multiple(array('task_attachment_id' => $t_v_files->task_attachment_id));
                        }
                    }
                    //delete into table.
                    $this->items_model->_table_name = "tbl_task_attachment"; // table name
                    $this->items_model->delete_multiple(array('task_id' => $v_taks->task_id));

                    //delete into table.
                    $this->items_model->_table_name = "tbl_tasks_timer"; // table name
                    $this->items_model->delete_multiple(array('task_id' => $v_taks->task_id));

                    $pin_info = $this->items_model->check_by(array('module_name' => 'tasks', 'module_id' => $v_taks->task_id), 'tbl_pinaction');
                    if (!empty($pin_info)) {
                        $this->items_model->_table_name = 'tbl_pinaction';
                        $this->items_model->delete_multiple(array('module_name' => 'tasks', 'module_id' => $v_taks->task_id));
                    }
                }
            }
            //save data into table.
            $this->items_model->_table_name = "tbl_task"; // table name
            $this->items_model->delete_multiple(array('leads_id' => $id));
            // get all proposal by leads id
            $proposal_info = $this->items_model->get_result(array('module' => 'leads', 'module_id' => $id), 'tbl_proposals');
            if (!empty($proposal_info)) {
                foreach ($proposal_info as $v_proposal) {
                    $this->items_model->_table_name = 'tbl_proposals_items';
                    $this->items_model->delete_multiple(array('proposals_id' => $v_proposal->proposals_id));

                    $this->items_model->_table_name = 'tbl_proposals';
                    $this->items_model->delete_multiple(array('proposals_id' => $v_proposal->proposals_id));
                }
            }
            $this->items_model->_table_name = 'tbl_reminders';
            $this->items_model->delete_multiple(array('module' => 'leads', 'module_id' => $id));

            $this->items_model->_table_name = 'tbl_pinaction';
            $this->items_model->delete_multiple(array('module_name' => 'leads', 'module_id' => $id));

            $this->items_model->_table_name = 'tbl_leads';
            $this->items_model->_primary_key = 'leads_id';
            $this->items_model->delete($id);

            $type = 'success';
            $message = lang('leads_deleted');
        } else {
            $type = 'error';
            $message = lang('there_in_no_value');
        }
        if (!empty($bulk)) {
            return (array("status" => $type, 'message' => $message));
        }
        echo json_encode(array("status" => $type, 'message' => $message));
        exit();
    }

    public function change_leads_status($lead_status_id)
    {
        $leads_id = $this->input->post('leads_id', true);
        foreach ($leads_id as $key => $id) {
            $data['index_no'] = $key + 1;
            //save data into table.
            $data['lead_status_id'] = $lead_status_id;
            $this->items_model->_table_name = 'tbl_leads';
            $this->items_model->_primary_key = 'leads_id';
            $this->items_model->save($data, $id);
        }
        $type = "success";
        $message = lang('update_leads');
        echo json_encode(array("status" => $type, "message" => $message));
        exit();
    }


    // Get subcategories by category id
    public function getLeadSubcategoriesByCatId($category_id=NULL)
    {
        $category_id=$this->input->post('category_id');
        if ($rows = $this->items_model->getAllSubcategoriesByCatId($category_id)) 
        {
            $data = json_encode($rows);
        } else {
            $data = json_encode(NULL);
        }
        echo $data;
        exit();
    }


    public function pickup($id = NULL)
    {
        if($id)
        {
            // check lead pickup by other call operator
            $newlead = $this->items_model->check_by(array('leads_id' => $id, 'lead_status' => 'New', 'call_operator_id' => NULL), 'tbl_leads');
            $pickedlead = $this->items_model->check_by(array('leads_id' => $id, 'lead_status' => 'Picked', 'call_operator_id' => $this->session->userdata('user_id')), 'tbl_leads');
            $reschlead = $this->items_model->check_by(array('leads_id' => $id, 'lead_status' => 'Rescheduled'), 'tbl_leads');
            if($newlead || $pickedlead || $reschlead)
            {
                $data['title'] = lang('leads');
                $data['active'] = 1;

                $data['leads_info'] = $this->items_model->check_by(array('leads_id' => $id), 'tbl_leads');
                $data['districts'] = $this->items_model->getAllDistrictsByStateId($data['leads_info']->state_id);
                $data['subcategories'] = $this->items_model->getAllSubcategoriesByCatId($data['leads_info']->lead_category_id);
                $data['states'] = $this->items_model->get_all_states();
                $data['categories'] = $this->items_model->get_all_lead_categories();
                $data['products'] = $this->items_model->get_all_products();

                $data['calloperators'] = $this->items_model->get_call_operators();
                $data['supplier'] = $this->items_model->get_manufacturer_list('Electricity');

                // update lead status
                $lead = array(
                    'lead_status' => 'Picked',
                    'rescheduled_time' => NULL,
                    'call_operator_id' => $this->session->userdata('user_id'),
                );
                $this->items_model->_table_name = 'tbl_leads';
                $this->items_model->_primary_key = 'leads_id';
                $return_id = $this->items_model->save($lead, $id);

                if(empty($pickedlead))
                {
                    // added entry in activity table
                    $action = lang('pickup_lead');
                    $activity = array(
                        'user' => $this->session->userdata('user_id'),
                        'module' => 'leads',
                        'module_field_id' => $id,
                        'activity' => $action,
                        'icon' => 'fa-rocket',
                        'link' => 'admin/leads/leads_details/' . $id,
                        'value1' => $data['leads_info']->first_name . ' ' . $data['leads_info']->middle_name . ' ' . $data['leads_info']->last_name 
                    );
                    $this->items_model->_table_name = 'tbl_activities';
                    $this->items_model->_primary_key = 'activities_id';
                    $this->items_model->save($activity);
                }

                $data['activities'] = $this->items_model->getAllActivities($id);

                $data['subview'] = $this->load->view('admin/leads/lead_support', $data, TRUE);
                $this->load->view('admin/_layout_main', $data); //page load
            }
            else
            {
                $type = 'error';
                $msg = "<strong style='color:#000'>" . $cate_data['lead_status'] . '</strong> This lead already picked by other call operator.';
                set_message($type, $msg);
                redirect('admin/dashboard');
            }
        }
    }

    // update status for call center
    public function update_status($id = NULL)  
    {  
        $edited = can_action('55', 'edited');
        if (!empty($edited) && !empty($id)) {
            $this->form_validation->set_rules('lead_status', lang('lead_status'),'trim|required');
            $this->form_validation->set_rules('comment', lang('comment'),'trim|required');
            if($this->form_validation->run() == true)
            {
                $this->items_model->_table_name = 'tbl_leads';
                $this->items_model->_primary_key = 'leads_id';
                $data = $this->items_model->array_from_post(array('lead_status'));
                $data['updated_by'] = $this->session->userdata('user_id');
                $data['updated_date'] = date('Y-m-d H:i:s');
                
                $return_id = $this->items_model->save($data, $id);
                if($return_id)
                {
                    if (!empty($id)) 
                    {
                        $id = $id;
                        $action = 'Changed Status';
                        $msg = lang('changed_the_leads_status');
                    } 
            
                    $activity = array(
                        'user' => $this->session->userdata('user_id'),
                        'module' => 'leads',
                        'module_field_id' => $id,
                        'activity' => $action,
                        'icon' => 'fa-rocket',
                        'link' => 'admin/leads/leads_details/' . $id,
                        'value1' => $this->input->post('comment')
                    );
                    $this->items_model->_table_name = 'tbl_activities';
                    $this->items_model->_primary_key = 'activities_id';
                    $this->items_model->save($activity);
                    // messages for user
                    $type = "success";
            
                    $message = $msg;
                    set_message($type, $message);
                    redirect('admin/dashboard');
                }
                else
                {
                    $error = lang('update_lead_error');
                }
            }

            if($this->form_validation->run() == false || empty($return_id))
            {
                $data['error'] = (validation_errors() ? validation_errors() : ($error) ? $error : '');
                $data['title'] = lang('leads');
                $data['active'] = 4;
                $data['leads_info'] = $this->items_model->check_by(array('leads_id' => $id), 'tbl_leads');
                $data['districts'] = $this->items_model->getAllDistrictsByStateId($data['leads_info']->state_id);
                $data['subcategories'] = $this->items_model->getAllSubcategoriesByCatId($data['leads_info']->lead_category_id);
                $data['states'] = $this->items_model->get_all_states();
                $data['categories'] = $this->items_model->get_all_lead_categories();
                $data['products'] = $this->items_model->get_all_products();

                $data['subview'] = $this->load->view('admin/leads/lead_support', $data, TRUE);
                $this->load->view('admin/_layout_main', $data);
            }            
        }        
    }   
    
    // update status for call center
    public function reschedule_leads($id = NULL)   
    {  
        $edited = can_action('55', 'edited');
        if (!empty($edited) && !empty($id)) {
            $this->form_validation->set_rules('rescheduled_time', lang('reschedule_time'),'trim|required');
            $this->form_validation->set_rules('rescheduled_reason', lang('reschedule_reason'),'trim|required');
            if($this->form_validation->run() == true)
            {
                $this->items_model->_table_name = 'tbl_leads';
                $this->items_model->_primary_key = 'leads_id';
                $data = $this->items_model->array_from_post(array('rescheduled_time'));
                $data['lead_status'] = 'Rescheduled';
                $data['updated_by'] = $this->session->userdata('user_id');
                $data['updated_date'] = date('Y-m-d H:i:s');
                $return_id = $this->items_model->save($data, $id);
                if($return_id)
                {
                    if (!empty($id)) 
                    {
                        $id = $id;
                        $action = 'Rescheduled Lead';
                        $msg = lang('rescheduled_the_leads');
                    } 

                    $activity = array(
                        'user' => $this->session->userdata('user_id'),
                        'module' => 'leads',
                        'module_field_id' => $id,
                        'activity' => $action,
                        'icon' => 'fa-rocket',
                        'link' => 'admin/leads/leads_details/' . $i,
                        'value1' => $this->input->post('rescheduled_reason')
                    );
                    $this->items_model->_table_name = 'tbl_activities';
                    $this->items_model->_primary_key = 'activities_id';
                    $this->items_model->save($activity);
                    // messages for user
                    $type = "success";            
                    $message = $msg;
                    set_message($type, $message);
                    redirect('admin/dashboard');
                }
                else
                {
                    $error = lang('update_lead_error');
                }
            }

            if($this->form_validation->run() == false || empty($return_id))
            {
                $data['error'] = (validation_errors() ? validation_errors() : ($error ? $error : ''));
                $data['title'] = lang('leads');
                $data['active'] = 2;
                $data['leads_info'] = $this->items_model->check_by(array('leads_id' => $id), 'tbl_leads');
                $data['districts'] = $this->items_model->getAllDistrictsByStateId($data['leads_info']->state_id);
                $data['subcategories'] = $this->items_model->getAllSubcategoriesByCatId($data['leads_info']->lead_category_id);
                $data['states'] = $this->items_model->get_all_states();
                $data['categories'] = $this->items_model->get_all_lead_categories();
                $data['products'] = $this->items_model->get_all_products();

                $data['subview'] = $this->load->view('admin/leads/lead_support', $data, TRUE);
                $this->load->view('admin/_layout_main', $data);
            }            
        }        
    }

    // reassign lead for call center
    public function reassign_leads($id = NULL)   
    {  
        $edited = can_action('55', 'edited');
        if (!empty($edited) && !empty($id)) {
            $this->form_validation->set_rules('call_center_id', lang('calloperator'),'trim|required');
            $this->form_validation->set_rules('reassign_reason', lang('reassign_reason'),'trim|required');
            if($this->form_validation->run() == true)
            {
                $this->items_model->_table_name = 'tbl_leads';
                $this->items_model->_primary_key = 'leads_id';
                $data = array(
                    'call_operator_id' => $this->input->post('call_center_id'),
                    'assigned_time' => date('Y-m-d H:i:s'),
                    'lead_status' => 'Reassigned',
                    'updated_by' => $this->session->userdata('user_id'),
                    'updated_date' => date('Y-m-d H:i:s')
                );
                
                $return_id = $this->items_model->save($data, $id);
                if($return_id)
                {
                    if (!empty($id)) 
                    {
                        $id = $id;
                        $action = 'Reassigned Lead';
                        $msg = lang('reassigned_the_lead');
                    } 
        
                    $activity = array(
                        'user' => $this->session->userdata('user_id'),
                        'module' => 'leads',
                        'module_field_id' => $id,
                        'activity' => $action,
                        'icon' => 'fa-rocket',
                        'link' => 'admin/leads/leads_details/' . $id,
                        'value1' => $this->input->post('reassign_reason')
                    );
                    $this->items_model->_table_name = 'tbl_activities';
                    $this->items_model->_primary_key = 'activities_id';
                    $this->items_model->save($activity);
                    // messages for user
                    $type = "success";            
                    $message = $msg;
                    set_message($type, $message);
                    redirect('admin/dashboard');
                }
                else
                {
                    $error = lang('update_lead_error');
                }
            }

            if($this->form_validation->run() == false || empty($return_id))
            {
                $data['error'] = (validation_errors() ? validation_errors() : ($error ? $error : ''));
                $data['title'] = lang('leads');
                $data['active'] = 3;
                $data['leads_info'] = $this->items_model->check_by(array('leads_id' => $id), 'tbl_leads');
                $data['districts'] = $this->items_model->getAllDistrictsByStateId($data['leads_info']->state_id);
                $data['subcategories'] = $this->items_model->getAllSubcategoriesByCatId($data['leads_info']->lead_category_id);
                $data['states'] = $this->items_model->get_all_states();
                $data['categories'] = $this->items_model->get_all_lead_categories();
                $data['products'] = $this->items_model->get_all_products();

                $data['subview'] = $this->load->view('admin/leads/lead_support', $data, TRUE);
                $this->load->view('admin/_layout_main', $data);
            }            
        }        
    }

    // Update or convert to customer for call center
    public function update_leads($id = NULL) 
    {  //echo '<pre>';print_r($_POST);die;
        $created = can_action('55', 'created');
        $edited = can_action('55', 'edited');
        if (!empty($created) || !empty($edited) && !empty($id)) 
        {
            $this->items_model->_table_name = 'tbl_leads';
            $this->items_model->_primary_key = 'leads_id';

            $this->form_validation->set_rules('first_name', lang('first_name'),'trim|required');
            $this->form_validation->set_rules('last_name', lang('last_name'),'trim|required');
            $this->form_validation->set_rules('mobile', lang('mobile'),'trim|required');
            if($this->input->post('is_appointment')=='on')
            {
                $this->form_validation->set_rules('appointment_address', lang('appointment_address'),'trim|required');
                $this->form_validation->set_rules('appointment_time', lang('appointment_time'),'trim|required');
            }
            if($this->form_validation->run() == true)
            {
                $data = $this->items_model->array_from_post(array('first_name', 'middle_name', 'last_name', 'email', 'mobile', 'address', 'state_id', 'district_id', 'city', 'pincode', 'lead_category_id', 'lead_subcategory_id', 'product_id', 'consumer_number', 'supplier', 'billing_unit', 'units', 'approx_unit_rate', 'approximate_bill_per_month', 'cust_message', 'telecaller_comment','lead_type'));

                $data['rescheduled_time'] = NULL;
                $data['is_appointment'] = $this->input->post('is_appointment')== 'on' ? 'Yes' : 'No';
                $data['updated_by'] = $this->session->userdata('user_id');
                $data['lead_status'] = 'Ready To Assign';
                $data['updated_date'] = date('Y-m-d H:i:s');

                if($this->input->post('units'))
                {
                    $data['calculation_type'] = 'unit';
                    $data['unit_type'] = 'Monthly';
                    $data['no_of_months'] = 1;
                }
                if(!empty($id))
                {
                    $lead = $this->items_model->check_by(array('leads_id' => $id), 'tbl_leads');
                }
                
                $return_id = $this->items_model->save($data, $id);
                if($return_id)
                {
                    if((!empty($this->input->post('state_id'))) && (!empty($this->input->post('district_id'))))
                    {
                        $year= date("Y");
                        $statePadded = sprintf("%02d", $this->input->post('state_id'));
                        $districtPadded = sprintf("%04d",$this->input->post('district_id'));
                        $idPadded = sprintf("%06d", $return_id);
                        $code=$year.$statePadded.$districtPadded.$idPadded;
                        $this->db->where('leads_id',$return_id);
                        $this->db->update('tbl_leads',array('lead_number' => $code));
                    }                  

                    if($this->input->post('is_appointment')=='on')
                    {
                        // added appointment 
                        $appointment = array(
                            'leads_id' => $id,
                            'appointment_time' => $this->input->post('appointment_time'),
                            'appointment_address' => $this->input->post('appointment_address'),
                            'status' => 'Scheduled',
                            'created_by' => $this->session->userdata('user_id'),
                            'created_date' => date('Y-m-d H:i:s'),
                        );

                        $this->items_model->_table_name = 'tbl_leads_appointment';
                        $this->items_model->_primary_key = 'id';
                        $appointment_id = $this->items_model->save($appointment);
                        if(!empty($appointment_id))
                        {
                            // send email to customer
                            $email_template = email_templates(array('email_group' => 'scheduled_appointment'), NULL, true);
                            $message = $email_template->template_body;
                            $subject = $email_template->subject;
                            $first_name = str_replace("{FIRST_NAME}", $this->input->post('first_name'), $message);
                            $middle_name = str_replace("{MIDDLE_NAME}", $this->input->post('middle_name'), $first_name);
                            $last_name = str_replace("{LAST_NAME}", $this->input->post('last_name'), $middle_name);
                            $message = str_replace("{SITE_NAME}", config_item('company_name'), $last_name);
                            $params['recipient'] = $this->input->post('email');
                            $params['subject'] = $subject;
                            $params['message'] = $message;
                            $this->items_model->send_email($params);

                            // added entry in activity table
                            $created_by = $this->session->userdata('user_id');
                            $action = 'Added appointment';
                            $activity = array(
                            'user' => $created_by,
                            'module' => 'leads',
                            'module_field_id' => $id,
                            'activity' => $action,
                            'icon' => 'fa-rocket',
                        );
                        $this->items_model->_table_name = 'tbl_activities';
                        $this->items_model->_primary_key = 'activities_id';
                        $this->items_model->save($activity);
                        }
                    }                    

                    //send email to customer
                    if($this->input->post('email'))
                    {
                        $email_template = email_templates(array('email_group' => 'appreciate_customer'), NULL, true);
                        $message = $email_template->template_body;
                        $subject = $email_template->subject;
                        
                        $first_name = str_replace("{FIRST_NAME}", $this->input->post('first_name'), $message);
                        $middle_name = str_replace("{MIDDLE_NAME}", $this->input->post('middle_name'), $first_name);
                        $last_name = str_replace("{LAST_NAME}", $this->input->post('last_name'), $middle_name);
                        $message = str_replace("{SITE_NAME}", config_item('company_name'), $last_name);
                        $params['recipient'] = $this->input->post('email');

                        $params['subject'] = $subject;
                        $params['message'] = $message;

                        $params['resourceed_file'] = '';
                        $this->items_model->send_email($params);
                    }
                    else if($this->input->post('mobile'))
                    {
                        // send appreciate sms
                        $mobileNum = $this->input->post('mobile');
                        $message = '%3C%23%3E Hello '.$this->input->post('first_name').', Thank you for contacting AFM Solar System. Our Team will get in touch with you shortly. pfH7juRe9tO';  
                        $otpResponse = $this->sendsmsGET($mobileNum, $message);
                    }
                    
                    // code for added entry in avtivity table
                    if (!empty($id)) 
                    {
                        $id = $id;
                        $action = 'Update lead details';
                        $description = 'not_update_leads';
                        $msg = lang('update_leads');
                    }

                    $activity = array(
                        'user' => $this->session->userdata('user_id'),
                        'module' => 'leads',
                        'module_field_id' => $id,
                        'activity' => $action,
                        'icon' => 'fa-rocket',
                        'link' => 'admin/leads/leads_details/' . $id,
                        'value1' => $data['first_name'] . ' ' . $data['middle_name'] . ' ' . $data['last_name'],
                        'value2' => 'Ready To Assign'
                    );
                    $this->items_model->_table_name = 'tbl_activities';
                    $this->items_model->_primary_key = 'activities_id';
                    $this->items_model->save($activity);

                    // messages for user
                    $type = "success";
                    $leads_info = $this->items_model->check_by(array('leads_id' => $id), 'tbl_leads');
                    if($this->input->post('save')=='update')
                    {
                        $message = $msg;
                        set_message($type, $message);
                        redirect('admin/dashboard');
                    }                
                }            
            } // end of validation if

            if($this->form_validation->run() == false || empty($return_id) || empty($clientid))
            {
                $data['error'] = (validation_errors() ? validation_errors() : ($error ? $error : ''));
                $data['title'] = lang('leads');
                $data['active'] = 1;
                $data['leads_info'] = $this->items_model->check_by(array('leads_id' => $id), 'tbl_leads');
                $data['districts'] = $this->items_model->getAllDistrictsByStateId($data['leads_info']->state_id);
                $data['subcategories'] = $this->items_model->getAllSubcategoriesByCatId($data['leads_info']->lead_category_id);
                $data['states'] = $this->items_model->get_all_states();
                $data['categories'] = $this->items_model->get_all_lead_categories();
                $data['products'] = $this->items_model->get_all_products();
                $data['supplier'] = $this->items_model->get_manufacturer_list('Electricity');

                $data['subview'] = $this->load->view('admin/leads/lead_support', $data, TRUE);
                $this->load->view('admin/_layout_main', $data);
            }
        } // end of edited if
    } //  end of function

    public function assign_to_salesperson()
    {
        $this->form_validation->set_rules('form_action', "Form action", 'required');
        $this->form_validation->set_rules('salesmanager_id', "Salesmanager", 'required');
        if ($this->form_validation->run() == true) 
        {
            if (!empty($_POST['ids'])) 
            {
                if ($this->input->post('form_action') == 'assign') 
                {
                    $salesmanager_id = $this->input->post('salesmanager_id');
                    $i = sizeof($_POST['ids']);
                    for ($r = 0; $r < $i; $r++) 
                    {
                        $leads_id = $_POST['ids'][$r];
                        $data1 = array(
                            'assigned_to' => $salesmanager_id,
                            'lead_status' => 'Assigned',
                            'updated_by' => $this->session->userdata('user_id'),
                            'updated_date' => date('Y-m-d H:i:s'),
                        );

                        // update leads table
                        $this->items_model->_table_name = 'tbl_leads';
                        $this->items_model->_primary_key = 'leads_id';
                        $id = $this->items_model->save($data1, $leads_id);
                        if($id)
                        {
                            // checked appointment is present for lead

                            $lead_info = $this->items_model->check_by(array('leads_id' => $leads_id), 'tbl_leads');

                            if($lead_info->is_appointment == 'Yes')
                            {
                                // added salesmanager id in appointment table
                                $app_data = array(
                                    'assigned_to' => $salesmanager_id,
                                    'updated_by' => $this->session->userdata('user_id'),
                                    'updated_date' => date('Y-m-d H:i:s')
                                );

                                $this->db->where('leads_id',$leads_id);
                                $this->db->update('tbl_leads_appointment',$app_data);
                            }
                            // added entry in activity table
                            $activity = 'Assigned To Salesmanager';
                            
                            // save into activities
                            $activities = array(
                                'user' => $this->session->userdata('user_id'),
                                'module' => 'leads',
                                'module_field_id' => $leads_id,
                                'activity' => $activity,
                                'icon' => 'fa-rocket',
                                'link' => 'admin/leads/leads_details/' . $leads_id,
                            );
                            // Update into tbl_project
                            $this->items_model->_table_name = "tbl_activities"; //table name
                            $this->items_model->_primary_key = "activities_id";
                            $this->items_model->save($activities);
                        }
                    }

                    // send email 
                    // get salesmanager details
                    $user_details = $this->items_model->check_by(array('user_id' => $salesmanager_id), 'tbl_users');
                
                    $account_details = $this->items_model->check_by(array('user_id' => $salesmanager_id), 'tbl_account_details');

                    $team_head_details = $this->items_model->check_by(array('user_id' => $this->session->userdata('user_id')), 'tbl_account_details');

                    $message = '<p>Hi ' . $account_details->fullname .',</p>';
                    if($i > 1)
                    {
                        $message .= '<p>Following leads have been assigned to you by <strong>'. $team_head_details->fullname .'</strong>.</p>';
                    }
                    else if($i == 1)
                    {
                        $message .= '<p>Following lead has been assigned to you by <strong>'. $team_head_details->fullname .'</strong>.</p>';
                    }
                    
                    $message .= '<table style="border:1px solid;" class="table table-striped DataTables bulk_table" id="DataTables" cellspacing="0" width="60%"><thead style="text-align:left;"><tr><th style="width:50px;border:1px solid;">Sr No</th style="border:1px solid;"><th style="border:1px solid;">Name</th><th style="border:1px solid;">Mobile</th></tr></thead><tbody>';
                    
                    for ($r = 0 , $sr = 1; $r < $i; $r++) 
                    {
                        
                        $leads_id = $_POST['ids'][$r];
                        $lead_details = $this->items_model->check_by(array('leads_id' => $leads_id), 'tbl_leads');
                        $message .= '<tr style="border:1px solid;"><td style="border:1px solid;">'.$sr.'</td><td style="border:1px solid;">'.$lead_details->first_name.' '. $lead_details->middle_name . ' ' . $lead_details->last_name .'</td><td style="border:1px solid;">'.$lead_details->mobile.'</td></tr>';
                        $sr++;
                    }
                    $message .='</tbody></table>';
                    
                    $config['mailtype'] = "html";
                    $this->load->library('email', $config);
                    $this->email->clear();
                    $this->email->from(config_item('company_email'), config_item('company_name'));
                    $this->email->to($user_details->email);
                    $this->email->subject('Leads assigned');
                    $this->email->message($message);
                    $send = $this->email->send();

                    if(!empty($user_details->notification_id))
                    {
                        // send notification to salesmanager
                        $message = 'You have been assigned ' . $i . 'leads';
                        $this->sendNotification($user_details->notification_id,$message,'Leads');
                    }

                    $type = 'success';
                    $message = 'Leads assigned to salesmanager successfully.';
                    $ss_data['last_fun'] = 'assigned';
                    $this->session->set_userdata($ss_data);
                    $data['action'] =3;
                    set_message($type, $message);
                    redirect('admin/leads/assigned_leads');
                }
            }
            else
            {
                $this->session->set_flashdata('error', "No record selected");
                redirect($_SERVER["HTTP_REFERER"]);
            }
        }
        else
        {
            $data['title'] = lang('all_leads');
            $data['error'] = (validation_errors() ? validation_errors() : ($error) ? $error : '');
            $data['active'] = 3;
            if($this->session->userdata('designations')=='Marketing Head')
            {
                $data['salesmanagers'] = $this->items_model->get_all_salesmanagers();
            }

            $data['states'] = $this->items_model->get_all_states();
            $data['categories'] = $this->items_model->get_all_lead_categories();
            $data['products'] = $this->items_model->get_all_products();
            $data['subview'] = $this->load->view('admin/leads/all_leads', $data, TRUE);
            
            $this->load->view('admin/_layout_main', $data); //page load
            redirect('admin/leads');
        }
    }

    // this function is used to show appointments 
    public function view_appointment($leads_id)
    {
        $data['leads_id'] = $leads_id;
        $data['leads_info'] = $this->items_model->check_by(array('leads_id' => $leads_id), 'tbl_leads');
        $data['appointment_info'] = $this->items_model->getappointment($leads_id);
        $data['modal_subview'] = $this->load->view('admin/leads/appointment_view', $data, FALSE);
        $this->load->view('admin/_layout_modal', $data);
    }   

    public function appointment()
    {
        //echo '<pre>';print_r($_POST);die;
        $edited = can_action('55', 'edited');
        if (!empty($edited)) 
        {
            $this->items_model->_table_name = 'tbl_leads_appointment';
            $this->items_model->_primary_key = 'id';

            if($this->input->post('save')=='add')
            {
                $data = array(
                    'leads_id' => $this->input->post('leads_id'),
                    'appointment_address' => trim($this->input->post('appointment_address')),
                    'appointment_time' => $this->input->post('appointment_time') ? $this->input->post('appointment_time') : NULL,
                    'status' => 'Scheduled',
                    'assigned_to' => $this->session->userdata('user_id'),
                    'created_by' => $this->session->userdata('user_id'),
                    'created_date' => date('Y-m-d H:i:s'),
                    'type' => $this->input->post('type'),
                );
                // check duplicate appointment
                $dupresult = false;
                if($type=='Agreement')
                {
                    $dupresult = $this->items_model->checkduplicate($data);
                }
                
                if(!$dupresult)
                {
                    $data['created_by'] = $this->session->userdata('user_id');
                    $data['created_date'] = date('Y-m-d H:i:s');
                    $data['assigned_to'] = $this->session->userdata('user_id');
                    if($this->input->post('type') == 'Follow Up')
                    {
                        $data['comment'] = $this->input->post('comment');
                    }
                    $id = $this->items_model->save($data);
                    if($id)
                    {
                        // update lead table
                        $this->items_model->_table_name = 'tbl_leads';
                        $this->items_model->_primary_key = 'leads_id';
                        $lead_data['is_appointment'] = 'Yes';
                        $returnid = $this->items_model->save($lead_data,$this->input->post('leads_id'));
                        if($returnid)
                        {
                            $created_by = $this->session->userdata('user_id');
                            $action = 'Appointment Added';
                            // added entry in activity table
                            $activity = array(
                                'user' => $created_by,
                                'module' => 'leads',
                                'module_field_id' => $this->input->post('leads_id'),
                                'activity' => $action,
                                'icon' => 'fa-rocket',
                            );
                            $this->items_model->_table_name = 'tbl_activities';
                            $this->items_model->_primary_key = 'activities_id';
                            $this->items_model->save($activity);
                            $type = "success";
                            $message = "Appointment added successfully";
                            set_message($type, $message);
                            if($this->input->post('type')=='Follow Up')
                            {
                                $data['active'] = 2;
                                $data['sub_active'] =1;
                                redirect('admin/leads/leads_details/'.$this->input->post('leads_id').'/2');
                            }
                            else
                            {
                                redirect('admin/leads/leads_details/'.$this->input->post('leads_id').'/3');
                            }                            
                        }
                    }
                }
                else
                {
                    $type = "error";
                    $message = "Appointment is already present";
                    set_message($type, $message);
                    redirect('admin/leads/leads_details/'.$this->input->post('leads_id').'/3');
                }
            }
            else if($this->input->post('save')=='update')
            {
                $data = array(
                        'appointment_time' => $this->input->post('appointment_time'),
                        'appointment_address' => $this->input->post('appointment_address'),
                        'status' => $this->input->post('status'),
                        'comment' => $this->input->post('comment'),
                        'updated_by' => $this->session->userdata('user_id'),
                        'updated_date' => date('Y-m-d H:i:s')
                    );
                $action = 'Appointment has been confirmed.';
                if($this->input->post('status') == 'Rescheduled')
                {
                    $data['assigned_to'] = NULL;
                }

                // update appointment
                $this->items_model->_table_name = 'tbl_leads_appointment';
                $this->items_model->_primary_key = 'id';
                $this->items_model->save($data,$this->input->post('appointmet_id'));

                if($this->input->post('status') == 'Rescheduled')
                {
                    $action = 'Appointment has been rescheduled.';
                    // insert new record
                    $appointment_data = array(
                        'leads_id' => $this->input->post('leads_id'),
                        'appointment_address' => trim($this->input->post('appointment_address')),
                        'appointment_time' => $this->input->post('appointment_time'),
                        'status' => 'Scheduled',
                        'assigned_to' => $this->session->userdata('user_id'),
                        'created_by' => $this->session->userdata('user_id'),
                        'created_date' => date('Y-m-d H:i:s')
                    );
                    $id = $this->items_model->save($appointment_data);
                }

                $created_by = $this->session->userdata('user_id');
                // added entry in activity table
                $activity = array(
                    'user' => $created_by,
                    'module' => 'leads',
                    'module_field_id' => $this->input->post('leads_id'),
                    'activity' => $action,
                    'icon' => 'fa-rocket',
                );
                $this->items_model->_table_name = 'tbl_activities';
                $this->items_model->_primary_key = 'activities_id';
                $this->items_model->save($activity);
                $type = "success";
                $message = "Appointment update successfully";
                set_message($type, $message);
                redirect('admin/leads/leads_details/'.$this->input->post('leads_id').'/3');
            }            
        }
    }

    public function quotationlist($id = NULL) 
    { 
        $data['title'] = lang('all_leads');
        if (!empty($id)) 
        {
            $data['active'] = 4;

            $data['leads_info'] = $this->items_model->check_by(array('leads_id' => $id), 'tbl_leads');
            $data['quotation_list'] = $this->quotations_model->getQuotationsById($id);
        } 
        $data['subview'] = $this->load->view('admin/leads/all_leads', $data, TRUE);
        $this->load->view('admin/_layout_main', $data); //page load
    }

    // this function is used to send quotation email to customer 
    public function share_quotation($quotations_id)
    {
        $quotation_info = $this->items_model->check_by(array('quotations_id' => $quotations_id), 'tbl_quotations');
        if(!empty($quotation_info))
        {
            $lead_info = $this->items_model->check_by(array('leads_id' => $quotation_info->lead_id), 'tbl_leads');

            if(!empty($lead_info->email))
            {
                $product = $this->items_model->get_any_field('tbl_products',array('product_id' => $lead_info->product_id),'name');
                // send email
                $email_template = email_templates(array('email_group' => 'send_quotation'), NULL, true);
                $message = $email_template->template_body;
                            $subject = $email_template->subject;
                $first_name = str_replace("{FIRST_NAME}", $lead_info->first_name, $message);
                $middle_name = str_replace("{MIDDLE_NAME}", $lead_info->middle_name, $first_name);
                $last_name = str_replace("{LAST_NAME}", $lead_info->last_name, $middle_name);
                $prod = str_replace("{PRODUCT}", $product, $last_name);
                $message = str_replace("{SITE_NAME}", config_item('company_name'), $prod);
                
                $params['resourceed_file'] = $quotation_info->quotation_path;
                $params['recipient'] = $lead_info->email;
                $params['subject'] = $subject;
                $params['message'] = $message;
                $response = $this->items_model->send_email($params);
                if($response)
                {
                    // update quotation status
                    $quo_array = array('quotation_status' => 'Shared','updated_by' => $this->session->userdata('user_id'),'updated_date' => date('Y-m-d H:i:s'));
                    $this->items_model->_table_name = 'tbl_quotations';
                    $this->items_model->_primary_key = 'quotations_id';
                    $quo_id = $this->items_model->save($quo_array,$quotations_id);

                    // update lead status
                    $lead_array = array('lead_status' => 'Quotation Sent','shared_quotation_id' => $quotations_id,'updated_by' => $this->session->userdata('user_id'),'updated_date' => date('Y-m-d H:i:s'));
                    $this->items_model->_table_name = 'tbl_leads';
                    $this->items_model->_primary_key = 'leads_id';
                    $leads_id = $this->items_model->save($lead_array,$lead_info->leads_id);

                    // added entry in activity table
                    $action = 'Sent Quotation';
                    $activity = array(
                        'user' => $this->session->userdata('user_id'),
                        'module' => 'leads',
                        'module_field_id' => $lead_info->leads_id,
                        'activity' => $action,
                        'icon' => 'fa-rocket',
                        'link' => '',
                    );
                    $this->items_model->_table_name = 'tbl_activities';
                    $this->items_model->_primary_key = 'activities_id';
                    $this->items_model->save($activity);

                    $type = "success";
                    $message = "Quotation email sent successfully.";
                }
                else
                {
                    $type = "error";
                    $message = "Something went wrong during send email,please try after sometime.";
                }
            }
        }
        else
        {
            $type = "error";
            $message = "Quotation information is not present.";
        }
        set_message($type, $message);
        redirect('admin/leads/quotationlist/'.$lead_info->leads_id);
    }

    // this function is used to request regenerate quotation
    public function updatestatus($leads_id,$status)
    {

        if($leads_id)
        {
            // update lead status to request for regenerate quotation
            if($status == 'Request_For_Regenerate_Quotation')
            {
                $status = 'Request For Regenerate Quotation';
            }
            else if($status == 'Approve')
            {
                $status = 'Quotation Approved';
            }
            $data = array(
                'lead_status' => $status,
                'updated_by' => $this->session->userdata('user_id'),
                'updated_date' => date('Y-m-d H:i:s')
            );
            $this->items_model->_table_name = 'tbl_leads';
            $this->items_model->_primary_key = 'leads_id';
            $id = $this->items_model->save($data,$leads_id);
            //echo '<pre>';print_r($status);die;
            if($id)  
            {
                $leads_info = $this->items_model->check_by(array('leads_id' => $leads_id), 'tbl_leads');
                if($status == 'Request For Regenerate Quotation')
                {
                    $quo['quotation_status'] = 'Rejected';
                }
                else if($status =='Quotation Approved')
                {
                    $quo['quotation_status'] = 'Approved';
                } 
                
                $this->items_model->_table_name = 'tbl_quotations';
                $this->items_model->_primary_key = 'quotations_id';
                $returnid = $this->items_model->save($quo,$leads_info->shared_quotation_id);
                if($returnid)
                {
                    // add entry in activity table
                    $action = $status;
                    $activity = array(
                        'user' => $this->session->userdata('user_id'),
                        'module' => 'leads',
                        'module_field_id' => $leads_id,
                        'activity' => $action,
                        'icon' => 'fa-rocket',
                        'link' => '',
                    );
                    $this->items_model->_table_name = 'tbl_activities';
                    $this->items_model->_primary_key = 'activities_id';
                    $this->items_model->save($activity);
                    $type = "success";
                    $message = "Status changed successfully.";
                }
                else
                {
                    // change status of lead
                    $leaddata = array(
                        'lead_status' => 'Quotation Sent',
                        'updated_by' => $this->session->userdata('user_id'),
                        'updated_date' => date('Y-m-d H:i:s')
                    );
                    $this->items_model->_table_name = 'tbl_leads';
                    $this->items_model->_primary_key = 'leads_id';
                    $id = $this->items_model->save($leaddata,$leads_id);
                    $type = "error";
                    $message = "Something went wrong during update quotation status,please try after sometime.";
                }
                   
            }
            else
            {
                $type = "error";
                $message = "Something went wrong during update lead status,please try after sometime.";
            }
        }
        set_message($type, $message);
        if($status == 'Quotation Approved')
        {
            redirect('admin/leads');
        }
        else
        {
            redirect('admin/leads/quotationlist/'.$leads_id);
        }
    }

    public function download_attachment($id,$type)
    {
        $this->items_model->_table_name = "tbl_leads";
        $leads_info = $this->items_model->get_by(array('leads_id' => $id), TRUE);
        if($type=='bill')
        {
            $file = $leads_info->attachment;
        }
        if (file_exists($file)) {
            $this->load->helper('download');
            force_download($file, NULL);
            exit();
        }
        $this->session->set_flashdata('error', "File not exist");
        redirect($_SERVER["HTTP_REFERER"]);
    }

    public function todays_appointments()
    {
        $data['title'] = lang('todays') . ' ' . lang('appointments');
        $view = can_action('171', 'view');
        if($view)
        {
            $data['subview'] = $this->load->view('admin/leads/todays_appointments', $data, TRUE);
            $this->load->view('admin/_layout_main', $data);
        }
    }

    public function getTodaysAppointments()
    {
        if ($this->input->is_ajax_request()) {
            $this->load->model('datatables');
            $this->datatables->table = 'tbl_leads_appointment';
            $custom_field = custom_form_table_search(5);
            $main_column = array('appointment_time', 'appointment_address','status');
            $action_array = array('id');
            $result = array_merge($main_column, $custom_field, $action_array);
            $this->datatables->column_order = $result;
            $this->datatables->column_search = $result;
            $this->datatables->order = array('id' => 'ASC');
            
            $user_id = $this->session->userdata('user_id');            
            $where = array('assigned_to' => $user_id,'DATE_FORMAT(appointment_time,"%Y-%m-%d")' => date('Y-m-d'));
            $fetch_data = $this->datatables->get_appointments($where);

            $data = array();
            foreach ($fetch_data as $_key => $v_appointment) {
                $action = null;
                if (!empty($v_appointment)) 
                {
                    $sub_array = array();
                    $leads_info = $this->items_model->check_by(array('leads_id' => $v_appointment->leads_id),'tbl_leads');
                    $name = $leads_info->first_name . ' ' .$leads_info->middle_name . ' ' . $leads_info->last_name;
                    $sub_array[] = $name;
                    $sub_array[] = $leads_info->mobile;
                    $sub_array[] = date('d-m-Y H:i:s',strtotime($v_appointment->appointment_time));
                    $sub_array[] = $v_appointment->appointment_address;
                    $sub_array[] = $v_appointment->status;
                    $data[] = $sub_array;
                }

            }
            render_table($data, $where);
        } else {
            redirect('admin/dashboard');
        }
    }

    public function convert_to_customer($id = NULL) 
    { 
        $data['title'] = lang('all_leads');
        if (!empty($id)) 
        {
            $data['active'] = 5;
            $data['dropzone'] = true;
            $data['leads_info'] = $this->items_model->check_by(array('leads_id' => $id), 'tbl_leads');
            $data['quotation_info'] = $this->items_model->check_by(array('quotations_id' => $data['leads_info']->shared_quotation_id), 'tbl_quotations');
        } 

        $data['subview'] = $this->load->view('admin/leads/all_leads', $data, TRUE);
        $this->load->view('admin/_layout_main', $data); //page load
    }

    public function converttocustomer()
    {
        $this->form_validation->set_rules('date', "Date", 'required');
        $this->form_validation->set_rules('amount', "Amount", 'required');
        $this->form_validation->set_rules('leads_id', "Lead Id", 'required');
        $this->form_validation->set_rules('payment_methods_id', "Payment Method", 'required');
        //echo '<pre>';print_r($_POST);die;
        if ($this->form_validation->run() == true) 
        {
            // Convert to customer
            $leads_id = $this->input->post('leads_id');
            $leads_info = $this->items_model->check_by(array('leads_id' => $leads_id),'tbl_leads');
            if($leads_info)
            {
                // check customer allready present in table or not
                $customer_info = $this->items_model->check_by(array('mobile' => $leads_info->mobile),'tbl_client');
                if(empty($customer_info))
                {
                    $not_present = true;
                    $password = $this->randomPassword(7);
                    $cust_data = array(
                        'first_name' => $leads_info->first_name,
                        'middle_name' => $leads_info->middle_name,
                        'last_name' => $leads_info->last_name,
                        'email' => $leads_info->email,
                        'mobile' => $leads_info->mobile,
                        'address' => $leads_info->address,
                        'state_id' => $leads_info->state_id,
                        'district_id' => $leads_info->district_id,
                        'city' => $leads_info->city,
                        'date_added' => date('Y-m-d H:i:s'),
                        'username' => $leads_info->mobile,
                        'password' => $this->hash($password),
                        'pincode' => $leads_info->pincode,
                        'created_by' => $this->session->userdata('user_id')
                    );
                    //echo '<pre>';print_r($cust_data);die;
                    // add customer in client table
                    $this->items_model->_table_name = 'tbl_client';
                    $this->items_model->_primary_key = "client_id";
                    $return_id = $this->items_model->save($cust_data);
                    if($return_id)
                    {
                        // create client number and update
                        $year= date("Y");
                        $statePadded = sprintf("%02d", $leads_info->state_id);
                        $districtPadded = sprintf("%04d",$leads_info->district_id);
                        $idPadded = sprintf("%06d", $return_id);
                        $code=$year.$statePadded.$districtPadded.$idPadded;
                        $cust_array = array('client_number' => $code );
                        // update client table 
                        $u_data['client_number'] = $code;
                        $rows = $this->items_model->save($u_data, $return_id);
                    }
                }
                else
                {
                    $not_present = false;
                    $return_id = $customer_info->client_id;
                }
                if($return_id)
                {
                    // create project
                    $project_data = array(
                        'client_id' => $return_id,
                        'lead_category_id' => $leads_info->lead_category_id,
                        'lead_subcategory_id' => $leads_info->lead_subcategory_id,
                        'product_id' => $leads_info->product_id,
                        'consumer_number' => $leads_info->consumer_number,
                        'supplier' => $leads_info->supplier,
                        'units' => $leads_info->units,
                        'unit_type' => $leads_info->unit_type,
                        'no_of_months' => $leads_info->no_of_months,
                        'approx_unit_rate' => $leads_info->approx_unit_rate,
                        'billing_unit' => $leads_info->billing_unit,
                        'created_by' => $leads_info->created_by,
                        'created_time' => date('Y-m-d H:i:s'),
                        'quotation_id' => $leads_info->shared_quotation_id
                    );

                    // add into project table
                    $this->items_model->_table_name = 'tbl_project';
                    $this->items_model->_primary_key = "project_id";
                    $project_id = $this->items_model->save($project_data);
                    if($project_id)
                    {
                        // add payment information in payment table
                        $payment_data = array(
                            'payment_date' => $this->input->post('date'),
                            'payment_method' => $this->input->post('payment_methods_id'),
                            'amount' => $this->input->post('amount'),
                            'reference_no' => $this->input->post('reference_no'),
                            'notes' => $this->input->post('notes'),
                            'cheque_no' => $this->input->post('cheque_no'),
                            'details' => $this->input->post('details'),
                            'project_id' => $project_id,
                            'created_by' => $this->session->userdata('user_id'),
                            'created_date' => date('Y-m-d H:i:s')
                        );

                        $upload_file = array();
                        $files = $this->input->post("files", true);
                        $target_path = getcwd() . "/uploads/payment/";
                        //process the fiiles which has been uploaded by dropzone
                        if (!empty($files) && is_array($files)) 
                        {
                            foreach ($files as $key => $file) 
                            {
                                if (!empty($file)) 
                                {
                                    $file_name = $this->input->post('file_name_' . $file, true);
                                    $new_file_name = move_temp_file($file_name, $target_path);
                                    $file_ext = explode(".", $new_file_name);
                                    $is_image = check_image_extension($new_file_name);
                                    $size = $this->input->post('file_size_' . $file, true) / 1000;
                                    if ($new_file_name) 
                                    {
                                        $up_data = array(
                                            "fileName" => $new_file_name,
                                            "path" => "uploads/payment/" . $new_file_name,
                                            "fullPath" => getcwd() . "/uploads/payment/" . $new_file_name,
                                            "ext" => '.' . end($file_ext),
                                            "size" => round($size, 2),
                                            "is_image" => $is_image,
                                        );
                                        array_push($upload_file, $up_data);
                                    }
                                }
                            }
                        }

                        $fileName = $this->input->post('fileName', true);
                        $path = $this->input->post('path', true);
                        $fullPath = $this->input->post('fullPath', true);
                        $size = $this->input->post('size', true);
                        $is_image = $this->input->post('is_image', true);

                        if (!empty($fileName)) 
                        {
                            foreach ($fileName as $key => $name) 
                            {
                                $old['fileName'] = $name;
                                $old['path'] = $path[$key];
                                $old['fullPath'] = $fullPath[$key];
                                $old['size'] = $size[$key];
                                $old['is_image'] = $is_image[$key];

                                array_push($upload_file, $old);
                            }
                        }
                        if (!empty($upload_file)) 
                        {
                            $payment_data['attachement'] = json_encode($upload_file);
                        } 
                        else 
                        {
                            $payment_data['attachement'] = null;
                        }
                        // added in payment table
                        $this->items_model->_table_name = 'tbl_payments';
                        $this->items_model->_primary_key = "payments_id";
                        $payment_id = $this->items_model->save($payment_data);
                        if($payment_id)
                        {
                            // change status of lead
                            // added project id in leads table
                            $this->items_model->_table_name = 'tbl_leads';
                            $this->items_model->_primary_key = "leads_id";
                            $leads_data['project_id'] = $project_id;
                            $leads_data['lead_status'] = 'Convert To Customer';
                            $rows = $this->items_model->save($leads_data, $leads_id);

                            $this->items_model->_table_name = 'tbl_project';
                            $this->items_model->_primary_key = "project_id";
                            $pr_data['paid'] = $this->input->post('amount');
                            $rows = $this->items_model->save($pr_data, $project_id);

                            if(!empty($leads_info->email) && !empty($not_present))
                            {
                                //send email to customer
                                $email_template = email_templates(array('email_group' => 'registration'));
                                $SITE_URL = str_replace("{SITE_URL}", base_url(), $email_template->template_body);
                                $username = str_replace("{USERNAME}", $leads_info->mobile, $SITE_URL);
                                $user_email = str_replace("{EMAIL}", $leads_info->email, $username);
                                $user_password = str_replace("{PASSWORD}", $password, $user_email);
                                $message = str_replace("{SITE_NAME}", config_item('company_name'), $user_password); 
                                $params['recipient'] = $leads_info->email;
                                $params['subject'] = '[ ' . config_item('company_name') . ' ]' . ' ' . $email_template->subject;
                                $params['message'] = $message;
                                $params['resourceed_file'] = '';
                                $this->items_model->send_email($params);
                            }

                            if(!empty($not_present))
                            {
                                // added customer activity
                                $action = ('activity_added_new_client');
                                $activities = array(
                                    'user' => $this->session->userdata('user_id'),
                                    'module' => 'client',
                                    'module_field_id' => $return_id,
                                    'activity' => $action,
                                    'icon' => 'fa-user',
                                    'value1' => $leads_info->first_name . ' ' . $leads_info->middle_name . ' ' . $leads_info->last_name
                                );
                                $this->items_model->_table_name = 'tbl_activities';
                                $this->items_model->_primary_key = "activities_id";
                                $this->items_model->save($activities);
                            }                           

                            // added activity for project
                            $action = 'activity_save_project';
                            // save into activities
                            $activity = array(
                                'user' => $this->session->userdata('user_id'),
                                'module' => 'projects',
                                'module_field_id' => $project_id,
                                'activity' => $action,
                                'icon' => 'fa-folder-open-o',
                                'link' => 'admin/projects/project_details/' . $project_id,
                            );
                            $this->items_model->_table_name = 'tbl_activities';
                            $this->items_model->_primary_key = 'activities_id';
                            $this->items_model->save($activity);

                            // added activity for payment
                            $activity = array(
                            'user' => $this->session->userdata('user_id'),
                            'module' => 'projects',
                            'module_field_id' => $project_id,
                            'activity' => ('activity_new_payment'),
                            'icon' => 'fa-shopping-cart',
                            'link' => 'admin/projects/project_details/' . $project_id,
                            'value1' => display_money($this->input->post('amount'), $currency->symbol),
                            );
                            $this->items_model->_table_name = 'tbl_activities';
                            $this->items_model->_primary_key = 'activities_id';
                            $this->items_model->save($activity);

                            $type = "success";
                            $message = "Customer created successfully.";
                            set_message($type, $message);
                            redirect('admin/leads');
                        }
                        else
                        {
                            // delete project and customer.
                            $this->items_model->_table_name = 'tbl_project';
                            $this->items_model->_primary_key = 'project_id';
                            $this->items_model->delete($project_id);

                            $this->items_model->_table_name = 'tbl_client';
                            $this->items_model->_primary_key = 'client_id';
                            $this->items_model->delete($return_id);

                            $type = "error";
                            $errer = "Something went wrong during inserting payment record,please try after sometime.";
                        }
                    }
                    else
                    {
                        // error in add project
                        if(!empty($not_present))
                        {
                            $this->items_model->_table_name = 'tbl_client';
                            $this->items_model->_primary_key = 'client_id';
                            $this->items_model->delete($return_id);
                        }                       

                        $type = "error";
                        $errer = "Something went wrong during inserting project record,please try after sometime.";
                    }
                }
                else
                {
                    // error in add customer
                    $type = "error";
                    $errer = "Something went wrong during inserting customer record,please try after sometime.";
                }
            }
            else
            {
                // empty lead
                $type = "error";
                $errer = "Lead information is not present in system.";
            }
            redirect($_SERVER['HTTP_REFERER']);
        }
        else
        {
            $data['error'] = validation_errors();
            $leads_id = $this->input->post('leads_id');
            $data['active'] = 5;
            $data['dropzone'] = true;
            $data['leads_info'] = $this->items_model->check_by(array('leads_id' => $leads_id), 'tbl_leads');
            $data['quotation_info'] = $this->items_model->check_by(array('quotations_id' => $data['leads_info']->shared_quotation_id), 'tbl_quotations');
            $data['subview'] = $this->load->view('admin/leads/all_leads', $data, TRUE);
            $this->load->view('admin/_layout_main', $data);
        }
    }

    public function hash($string)
    {
        return hash('sha512', $string . config_item('encryption_key'));
    }
    
    public function assigned_leads()
    {
        $data['title'] = lang('all_leads');
        $data['dropzone'] = true;
       
        if($this->session->userdata('designations')=='Marketing Head')
        {
            $data['salesmanagers'] = $this->items_model->get_all_salesmanagers();
        }
        if(!empty($active))
        {
            $data['active'] = $active;
        }
        
        $data['subview'] = $this->load->view('admin/leads/assigned_leads', $data, TRUE);
        $this->load->view('admin/_layout_main', $data); //page load
    }

    public function get_assigned_list($filterBy = null, $search_by = null)
    {
        if ($this->input->is_ajax_request()) {
            $this->load->model('datatables');
            $this->datatables->table = 'tbl_leads';
            $custom_field = custom_form_table_search(5);
            $main_column = array('first_name', 'middle_name', 'last_name', 'mobile', 'city', 'address','lead_status');
            $action_array = array('leads_id');
            $result = array_merge($main_column, $custom_field, $action_array);
            $this->datatables->column_order = $result;
            $this->datatables->column_search = $result;
            $this->datatables->order = array('leads_id' => 'ASC');

            $where = array();
                      
            if(empty($where))
            {
                $where = array('type' => 'Lead');
            }
            $status = array('Ready To Assign','Assigned','Ready To Reassign');
            $where_in = array('lead_status', $status);

            $fetch_data = $this->datatables->get_leads($filterBy, $search_by, $where,NULL,$where_in);

            $data = array();

            $edited = can_action('55', 'edited');
            $deleted = can_action('55', 'deleted');
            foreach ($fetch_data as $_key => $v_leads) {
                $action = null;
                if (!empty($v_leads)) {

                    $sub_array = array();
                    if (!empty($deleted)) {
                        $sub_array[] = '<div class="checkbox c-checkbox" ><label class="needsclick"> <input name="ids[]" value="' . $v_leads->leads_id . '" type="checkbox"><span class="fa fa-check"></span></label></div>';
                    }
                    $name = null;
                    $name .= '<a class="text-info" href="' . base_url() . 'admin/leads/leads_details/' . $v_leads->leads_id . '">' . $v_leads->first_name . ' ' .$v_leads->middle_name . ' ' . $v_leads->last_name . '</a>';                 
                    $sub_array[] = $name;
                    $sub_array[] = $v_leads->mobile;
                    if (is_numeric($v_leads->city))
                    {
                        $city_info = $this->items_model->check_by(array('city_id' => $v_leads->city),'tbl_cities');
                        $sub_array[] = $city_info->name;
                    }
                    else
                    {
                        $sub_array[] = $v_leads->city;
                    }

                    $sub_array[] = $v_leads->address;
                    $sub_array[] = $v_leads->lead_status;

                    if(!empty($v_leads->assigned_to))
                    {
                        // get salesmanagers name
                        $salesmanager = $this->items_model->check_by(array('user_id' => $v_leads->assigned_to),'tbl_account_details');
                        $sub_array[] = $salesmanager->fullname;
                    }
                    else
                    {
                        $sub_array[] = '-';
                    }
                    
                    if (!empty($edited)) 
                    {
                        $action .= btn_edit('admin/leads/index/' . $v_leads->leads_id) . ' ';
                    }
                        
                    if (!empty($deleted)) 
                    {
                        $action .= ajax_anchor(base_url("admin/leads/delete_leads/$v_leads->leads_id"), "<i class='btn btn-xs btn-danger fa fa-trash-o'></i>", array("class" => "", "title" => lang('delete'), "data-fade-out-on-success" => "#table_" . $_key)) . ' ';
                    }            
                    $sub_array[] = $action;
                    $data[] = $sub_array;
                }

            }
            render_table($data, $where, $where_in, NULL);
        } else {
            redirect('admin/dashboard');
        }
    }
}