<?php

if (!defined('BASEPATH'))
	exit('No direct script access allowed');

class Masters extends Admin_Controller
{

	public function __construct()
	{
		parent::__construct();
		$this->load->model('masters_model');
	}

	public function manufacturers($id = NULL)
	{
		$data['title'] = lang('all') . ' ' . lang('manufacturers');
		if (!empty($id) && is_numeric($id)) {
			$data['active'] = 2;
			$edited = can_action('165', 'edited');
			if (!empty($edited)) {
				$data['manufacturer_info'] = $this->masters_model->check_by(array('manufacturer_id' => $id), 'tbl_manufacturer');
			}
		} else {
			$data['active'] = 1;
		}
		$data['subview'] = $this->load->view('admin/masters/manufacturer_list', $data, TRUE);
		$this->load->view('admin/_layout_main', $data); //page load
	}

	public function manufacturerList()
	{
		if ($this->input->is_ajax_request()) {
			$this->load->model('datatables');
			$this->datatables->table = 'tbl_manufacturer';
			$this->datatables->column_order = array('name', 'type','manufacturer_id');
			$this->datatables->column_search = array('name', 'type','manufacturer_id');
			$this->datatables->order = array('manufacturer_id' => 'desc');
			$where = "type='Panel' or type='Inverter'";
			$fetch_data = make_datatables($where);
			//print_r($fetch_data);die;
			$data = array();

			$edited = can_action('167', 'edited');
			$deleted = can_action('167', 'deleted');
			foreach ($fetch_data as $_key => $v_items) {
				$action = null;
				$sub_array = array();
				$sub_array[] = $v_items->name;
				$sub_array[] = $v_items->type;
				
				
				if (!empty($edited)) {
					$action .= btn_edit('admin/masters/edit_manufacturer/' . $v_items->manufacturer_id) . ' ';
				}
				if (!empty($deleted)) {
					$action .= ajax_anchor(base_url("admin/masters/delete_manufacturer/$v_items->manufacturer_id"), "<i class='btn btn-xs btn-danger fa fa-trash-o'></i>", array("class" => "", "title" => lang('delete'), "data-fade-out-on-success" => "#table_" . $_key)) . ' ';
				}
				$sub_array[] = $action;
				$data[] = $sub_array;
			}
			render_table($data,$where);
		} else {
			redirect('admin/dashboard');
		}
	}

	public function new_manufacturer()
	{
		$data['title'] = lang('new') . ' ' . lang('manufacturer');
		$data['permission_user'] = $this->masters_model->all_permission_user('151');
		$data['subview'] = $this->load->view('admin/masters/new_manufacturer', $data, FALSE);
		$this->load->view('admin/_layout_modal', $data);
	}

	public function edit_manufacturer($id = NULL)
	{
		$data['title'] = lang('all') . ' ' . lang('manufacturer');
		if (!empty($id) && is_numeric($id)) {
			$data['active'] = 2;
			$edited = can_action('151', 'edited');
			if (!empty($edited)) {
				$data['manufacturer_info'] = $this->masters_model->check_by(array('manufacturer_id' => $id), 'tbl_manufacturer');
			}
		} else {
			$data['active'] = 1;
		}
		$data['subview'] = $this->load->view('admin/masters/manufacturer_list', $data, TRUE);
		$this->load->view('admin/_layout_main', $data); //page load
	}
	public function saved_manufacturer($id = NULL)
	{
		$created = can_action('167', 'created');
		$edited = can_action('167', 'edited');
		
		if (!empty($created) && !empty($edited)) {

			if (!empty($id) && $id == 'inline') {
				$id = null;
				$inline = true;
			}
			$this->masters_model->_table_name = 'tbl_manufacturer';
			$this->masters_model->_primary_key = 'manufacturer_id';

			$data = $this->masters_model->array_from_post(array('name', 'type'));
			// update root category
			$where = array('name' => $data['name'], 'type' => $data['type']);
			// duplicate value check in DB
			if (!empty($id)) {
				// if id exist in db update data
				$manufacturer_id = array('manufacturer_id !=' => $id);
			} else {
				// if id is not exist then set id as null
				$manufacturer_id = null;
			}
			// check whether this input data already exist or not
			$check_items = $this->masters_model->check_update('tbl_manufacturer', $where, $manufacturer_id);
			if (!empty($check_items)) {
				// if input data already exist show error alert
				// massage for user
				$type = 'error';
				$msg = "<strong style='color:#000'>" . $data['name'] . '</strong>  ' . lang('already_exist');
			} else {
				// save and update query
				$permission = $this->input->post('permission', true);
				
				$return_id = $this->masters_model->save($data, $id);

				save_custom_field(19, $id);

				if (!empty($id)) {
					$id = $id;
					$action = 'update_manufacturer';
					$msg = lang('update_manufacturer');
				} else {
					$id = $return_id;
					$action = 'save_manufacturer';
					$msg = lang('save_manufacturer');
				}
				$activity = array(
				'user' => $this->session->userdata('user_id'),
				'module' => 'manufacturer',
				'module_field_id' => $id,
				'activity' => $action,
				'icon' => 'fa-circle-o',
				'value1' => $data['name']
				);
				$this->masters_model->_table_name = 'tbl_activities';
				$this->masters_model->_primary_key = 'activities_id';
				$this->masters_model->save($activity);
				// messages for user
				$type = "success";
			}
			if (!empty($inline)) {
				if (!empty($return_id)) {
					$result = array(
					'id' => $id,
					'name' => $data['name'],
					'status' => $type,
					'message' => $msg,
					);
				} else {
					$result = array(
					'status' => $type,
					'message' => $msg,
					);
				}
				echo json_encode($result);
				exit();
			} else {
				$message = $msg;
				set_message($type, $message);
			}
		}
		redirect('admin/masters/manufacturers');
	}

	public function delete_manufacturer($id)
	{
		$manufacturer_info = $this->masters_model->check_by(array('manufacturer_id' => $id), 'tbl_manufacturer');
		$activity = array(
		'user' => $this->session->userdata('user_id'),
		'module' => 'manufacturer',
		'module_field_id' => $id,
		'activity' => 'manufacturer_deleted',
		'icon' => 'fa-circle-o',
		'value1' => $manufacturer_info->name
		);
		$this->masters_model->_table_name = 'tbl_activities';
		$this->masters_model->_primary_key = 'activities_id';
		$this->masters_model->save($activity);

		$this->masters_model->_table_name = 'tbl_manufacturer';
		$this->masters_model->_primary_key = 'manufacturer_id';
		$this->masters_model->delete($id);
		$type = 'success';
		$message = lang('manufacturer_deleted');
		echo json_encode(array("status" => $type, 'message' => $message));
		exit();
	}
	
	public function products($id = NULL)
	{
		$data['title'] = lang('all') . ' ' . lang('products');
		if (!empty($id) && is_numeric($id)) {
			$data['active'] = 2;
			$edited = can_action('165', 'edited');
			if (!empty($edited)) {
				$data['product_info'] = $this->masters_model->check_by(array('product_id' => $id), 'tbl_products');
			}
		} else {
			$data['active'] = 1;
		}
		$data['subview'] = $this->load->view('admin/masters/product_list', $data, TRUE);
		$this->load->view('admin/_layout_main', $data); //page load
	}

	public function productList()
	{
		if ($this->input->is_ajax_request()) {
			$this->load->model('datatables');
			$this->datatables->table = 'tbl_products';
			$this->datatables->column_order = array('name', 'status');
			$this->datatables->column_search = array('name', 'status');
			$this->datatables->order = array('product_id' => 'desc');

			$fetch_data = make_datatables();
			//print_r($fetch_data);die;
			$data = array();

			$edited = can_action('167', 'edited');
			$deleted = can_action('167', 'deleted');
			foreach ($fetch_data as $_key => $v_items) {
				$action = null;
				$sub_array = array();
				$sub_array[] = $v_items->name;
				$sub_array[] = $v_items->status;


				if (!empty($edited)) {
					$action .= btn_edit('admin/masters/edit_product/' . $v_items->product_id) . ' ';
				}
				if (!empty($deleted)) {
					$action .= ajax_anchor(base_url("admin/masters/delete_product/$v_items->product_id"), "<i class='btn btn-xs btn-danger fa fa-trash-o'></i>", array("class" => "", "title" => lang('delete'), "data-fade-out-on-success" => "#table_" . $_key)) . ' ';
				}
				$sub_array[] = $action;
				$data[] = $sub_array;
			}
			render_table($data);
		} else {
			redirect('admin/dashboard');
		}
	}

	public function new_product()
	{
		$data['title'] = lang('new') . ' ' . lang('product');
		$data['permission_user'] = $this->masters_model->all_permission_user('151');
		$data['subview'] = $this->load->view('admin/masters/new_product', $data, FALSE);
		$this->load->view('admin/_layout_modal', $data);
	}

	public function edit_product($id = NULL)
	{
		$data['title'] = lang('all') . ' ' . lang('product');
		if (!empty($id) && is_numeric($id)) {
			$data['active'] = 2;
			$edited = can_action('151', 'edited');
			if (!empty($edited)) {
				$data['product_info'] = $this->masters_model->check_by(array('product_id' => $id), 'tbl_products');
			}
		} else {
			$data['active'] = 1;
		}
		$data['subview'] = $this->load->view('admin/masters/product_list', $data, TRUE);
		$this->load->view('admin/_layout_main', $data); //page load
	}
	public function saved_product($id = NULL)
	{
		$created = can_action('167', 'created');
		$edited = can_action('167', 'edited');
		if (!empty($created) && !empty($edited)) {

			if (!empty($id) && $id == 'inline') {
				$id = null;
				$inline = true;
			}
			$this->masters_model->_table_name = 'tbl_products';
			$this->masters_model->_primary_key = 'product_id';
			//print_r($_POST);die;
			$data = $this->masters_model->array_from_post(array('name', 'status'));
			if (isset($_POST['scope_of_work'])) {
				$data['scope_of_work'] = json_encode($_POST['scope_of_work']);
			}
			// update root category
			$where = array('name' => $data['name'], 'status' => $data['status']);
			// duplicate value check in DB
			if (!empty($id)) {
				// if id exist in db update data
				$product_id = array('product_id !=' => $id);
			} else {
				// if id is not exist then set id as null
				$product_id = null;
			}
			// check whether this input data already exist or not
			$check_items = $this->masters_model->check_update('tbl_products', $where, $product_id);
			if (!empty($check_items)) {
				// if input data already exist show error alert
				// massage for user
				$type = 'error';
				$msg = "<strong style='color:#000'>" . $data['name'] . '</strong>  ' . lang('already_exist');
			} else {
				// save and update query
				$permission = $this->input->post('permission', true);

				$return_id = $this->masters_model->save($data, $id);

				save_custom_field(19, $id);

				if (!empty($id)) {
					$id = $id;
					$action = 'update_product';
					$msg = lang('update_product');
				} else {
					$id = $return_id;
					$action = 'save_product';
					$msg = lang('save_product');
				}
				$activity = array(
				'user' => $this->session->userdata('user_id'),
				'module' => 'product',
				'module_field_id' => $id,
				'activity' => $action,
				'icon' => 'fa-circle-o',
				'value1' => $data['name']
				);
				$this->masters_model->_table_name = 'tbl_activities';
				$this->masters_model->_primary_key = 'activities_id';
				$this->masters_model->save($activity);
				// messages for user
				$type = "success";
			}
			if (!empty($inline)) {
				if (!empty($return_id)) {
					$result = array(
					'id' => $id,
					'name' => $data['name'],
					'status' => $type,
					'message' => $msg,
					);
				} else {
					$result = array(
					'status' => $type,
					'message' => $msg,
					);
				}
				echo json_encode($result);
				exit();
			} else {
				$message = $msg;
				set_message($type, $message);
			}
		}
		redirect('admin/masters/products');
	}

	public function delete_product($id)
	{
		$product_info = $this->masters_model->check_by(array('product_id' => $id), 'tbl_products');
		$activity = array(
		'user' => $this->session->userdata('user_id'),
		'module' => 'product',
		'module_field_id' => $id,
		'activity' => 'product_deleted',
		'icon' => 'fa-circle-o',
		'value1' => $product_info->name
		);
		$this->masters_model->_table_name = 'tbl_activities';
		$this->masters_model->_primary_key = 'activities_id';
		$this->masters_model->save($activity);

		$this->masters_model->_table_name = 'tbl_products';
		$this->masters_model->_primary_key = 'product_id';
		$this->masters_model->delete($id);
		$type = 'success';
		$message = lang('product_deleted');
		echo json_encode(array("status" => $type, 'message' => $message));
		exit();
	}
	
	public function esuppliers($id = NULL)
	{
		$data['title'] = lang('electricity') . ' ' . lang('suppliers');
		if (!empty($id) && is_numeric($id)) {
			$data['active'] = 2;
			$edited = can_action('165', 'edited');
			echo '<pre>';print_r($edited);die;
			if (!empty($edited)) {
				$data['supplier_info'] = $this->masters_model->check_by(array('manufacturer_id' => $id), 'tbl_manufacturer');
			}
		} else {
			$data['active'] = 1;
		}
		$data['subview'] = $this->load->view('admin/masters/esupplier_list', $data, TRUE);
		$this->load->view('admin/_layout_main', $data); //page load
	}
	public function esupplierList()
	{
		if ($this->input->is_ajax_request()) {
			$this->load->model('datatables');
			$this->datatables->table = 'tbl_manufacturer';
			$this->datatables->column_order = array('name');
			$this->datatables->column_search = array('name');
			$this->datatables->order = array('manufacturer_id' => 'desc');
			$where = "type = 'Electricity'";

			$fetch_data = make_datatables($where);
			//print_r($fetch_data);die;
			$data = array();

			$edited = can_action('168', 'edited');
			$deleted = can_action('168', 'deleted');
			foreach ($fetch_data as $_key => $v_items) {
				$action = null;
				$sub_array = array();
				$sub_array[] = $v_items->name;
				$sub_array[] = $v_items->type;


				if (!empty($edited)) {
					$action .= btn_edit('admin/masters/edit_supplier/' . $v_items->manufacturer_id) . ' ';
				}
				if (!empty($deleted)) {
					$action .= ajax_anchor(base_url("admin/masters/delete_manufacturer/$v_items->manufacturer_id"), "<i class='btn btn-xs btn-danger fa fa-trash-o'></i>", array("class" => "", "title" => lang('delete'), "data-fade-out-on-success" => "#table_" . $_key)) . ' ';
				}
				$sub_array[] = $action;
				$data[] = $sub_array;
			}
			render_table($data,$where);
		} else {
			redirect('admin/dashboard');
		}
	}
	
	public function saved_suppliers($id = NULL)
	{
		$created = can_action('168', 'created');
		$edited = can_action('168', 'edited');

		if (!empty($created) && !empty($edited)) {

			if (!empty($id) && $id == 'inline') {
				$id = null;
				$inline = true;
			}
			$this->masters_model->_table_name = 'tbl_manufacturer';
			$this->masters_model->_primary_key = 'manufacturer_id';

			$data = $this->masters_model->array_from_post(array('name', 'type'));
			// update root category
			$where = array('name' => $data['name'], 'type' => $data['type']);
			// duplicate value check in DB
			if (!empty($id)) {
				// if id exist in db update data
				$manufacturer_id = array('manufacturer_id !=' => $id);
			} else {
				// if id is not exist then set id as null
				$manufacturer_id = null;
			}
			// check whether this input data already exist or not
			$check_items = $this->masters_model->check_update('tbl_manufacturer', $where, $manufacturer_id);
			if (!empty($check_items)) {
				// if input data already exist show error alert
				// massage for user
				$type = 'error';
				$msg = "<strong style='color:#000'>" . $data['name'] . '</strong>  ' . lang('already_exist');
			} else {
				// save and update query
				$permission = $this->input->post('permission', true);

				$return_id = $this->masters_model->save($data, $id);

				save_custom_field(19, $id);

				if (!empty($id)) {
					$id = $id;
					$action = 'update_supplier';
					$msg = lang('update_supplier');
				} else {
					$id = $return_id;
					$action = 'save_supplier';
					$msg = lang('save_supplier');
				}
				$activity = array(
				'user' => $this->session->userdata('user_id'),
				'module' => 'supplier',
				'module_field_id' => $id,
				'activity' => $action,
				'icon' => 'fa-circle-o',
				'value1' => $data['name']
				);
				$this->masters_model->_table_name = 'tbl_activities';
				$this->masters_model->_primary_key = 'activities_id';
				$this->masters_model->save($activity);
				// messages for user
				$type = "success";
			}
			if (!empty($inline)) {
				if (!empty($return_id)) {
					$result = array(
					'id' => $id,
					'name' => $data['name'],
					'status' => $type,
					'message' => $msg,
					);
				} else {
					$result = array(
					'status' => $type,
					'message' => $msg,
					);
				}
				echo json_encode($result);
				exit();
			} else {
				$message = $msg;
				set_message($type, $message);
			}
		}
		redirect('admin/masters/esuppliers');
	}
	public function edit_supplier($id = NULL)
	{
		$data['title'] = lang('Edit') . ' ' . lang('supplier');
		if (!empty($id) && is_numeric($id)) {
			$data['active'] = 2;
			$edited = can_action('168', 'edited');
			if (!empty($edited)) {
				$data['manufacturer_info'] = $this->masters_model->check_by(array('manufacturer_id' => $id), 'tbl_manufacturer');
			}
		} else {
			$data['active'] = 1;
		}
		$data['subview'] = $this->load->view('admin/masters/esupplier_list', $data, TRUE);
		$this->load->view('admin/_layout_main', $data); //page load
	}
	
	public function categories($id = NULL)
	{
		$data['title'] = lang('all') . ' ' . lang('categories');
		if (!empty($id) && is_numeric($id)) {
			$data['active'] = 2;
			$edited = can_action('169', 'edited');
			if (!empty($edited)) {
				$data['subcategory_info'] = $this->masters_model->check_by(array('id' => $id), $_REQUEST);
			}
		} else {
			$data['active'] = 1;
		}
		
		$data['subview'] = $this->load->view('admin/masters/category_list', $data, TRUE);
		$this->load->view('admin/_layout_main', $data); //page load
	}

	


	public function new_supplier()
    {
        $data['title'] = lang('new_supplier');
        $data['subview'] = $this->load->view('admin/masters/new_supplier', $data, FALSE);
        $this->load->view('admin/_layout_modal', $data);
    }

    public function add_new_supplier()
    {
        
        
            $data = $this->masters_model->array_from_post(array('name', 'type'));
			$this->masters_model->_table_name = 'tbl_manufacturer';
			$this->masters_model->_primary_key = 'manufacturer_id';
			$id = $this->masters_model->save($data);

			if (!empty($id)) 
			{
				$id = $id;
				$action = 'save_supplier';
				$msg = lang('save_supplier');
			} 
			
			$activity = array(
				'user' => $this->session->userdata('user_id'),
				'module' => 'supplier',
				'module_field_id' => $id,
				'activity' => $action,
				'icon' => 'fa-circle-o',
				'value1' => $data['name']
			);
			$this->masters_model->_table_name = 'tbl_activities';
			$this->masters_model->_primary_key = 'activities_id';
			$this->masters_model->save($activity);
            //            messages for user
            $type = "success";
            $message = lang('supplier_added');
            set_message($type, $message);
        
        if (!empty($id)) {
            $result = array(
                'id' => $id,
                'name' => $data['name'],
                'status' => $type,
                'message' => $message,
            );
        } else {
            $result = array(
                'status' => 'error',
                'message' => lang('there_in_no_value'),
            );
        }
        echo json_encode($result);
        exit();
    }

	public function categoryList()
	{
		if ($this->input->is_ajax_request()) {
			$this->load->model('datatables');
			$this->datatables->table = 'tbl_customer_categories';
			$this->datatables->column_order = array('name','status');
			$this->datatables->column_search = array('name','status');
			$this->datatables->order = array('id' => 'desc');
			//$where = "type = 'Electricity'";

			$fetch_data = make_datatables();
			//print_r($fetch_data);die;
			$data = array();

			$edited = can_action('169', 'edited');
			$deleted = can_action('169', 'deleted');
			foreach ($fetch_data as $_key => $v_items) {
				$action = null;
				$sub_array = array();
				$sub_array[] = $v_items->name;
				$sub_array[] = $v_items->status;


				if (!empty($edited)) {
					$action .= btn_edit('admin/masters/edit_category/' . $v_items->id) . ' ';
				}
				if (!empty($deleted)) {
					$action .= ajax_anchor(base_url("admin/masters/delete_category/$v_items->id"), "<i class='btn btn-xs btn-danger fa fa-trash-o'></i>", array("class" => "", "title" => lang('delete'), "data-fade-out-on-success" => "#table_" . $_key)) . ' ';
				}
				$sub_array[] = $action;
				$data[] = $sub_array;
			}
			render_table($data);
		} else {
			redirect('admin/dashboard');
		}
	}

	public function saved_categories($id = NULL)
	{
		$created = can_action('169', 'created');
		$edited = can_action('169', 'edited');

		if (!empty($created) && !empty($edited)) {

			if (!empty($id) && $id == 'inline') {
				$id = null;
				$inline = true;
			}
			$this->masters_model->_table_name = 'tbl_customer_categories';
			$this->masters_model->_primary_key = 'id';

			$data = $this->masters_model->array_from_post(array('name', 'status'));
			// update root category
			$where = array('name' => $data['name'], 'status' => $data['status']);
			// duplicate value check in DB
			if (!empty($id)) {
				// if id exist in db update data
				$id = array('id !=' => $id);
			} else {
				// if id is not exist then set id as null
				$id = null;
			}
			// check whether this input data already exist or not
			$check_items = $this->masters_model->check_update('tbl_customer_categories', $where, $id);
			if (!empty($check_items)) {
				// if input data already exist show error alert
				// massage for user
				$type = 'error';
				$msg = "<strong style='color:#000'>" . $data['name'] . '</strong>  ' . lang('already_exist');
			} else {
				// save and update query
				$permission = $this->input->post('permission', true);

				$return_id = $this->masters_model->save($data, $id);

				save_custom_field(19, $id);

				if (!empty($id)) {
					$id = $id;
					$action = 'update_category';
					$msg = lang('update_category');
				} else {
					$id = $return_id;
					$action = 'save_category';
					$msg = lang('save_category');
				}
				$activity = array(
				'user' => $this->session->userdata('user_id'),
				'module' => 'customer_category',
				'module_field_id' => $id,
				'activity' => $action,
				'icon' => 'fa-circle-o',
				'value1' => $data['name']
				);
				$this->masters_model->_table_name = 'tbl_activities';
				$this->masters_model->_primary_key = 'activities_id';
				$this->masters_model->save($activity);
				// messages for user
				$type = "success";
			}
			if (!empty($inline)) {
				if (!empty($return_id)) {
					$result = array(
					'id' => $id,
					'name' => $data['name'],
					'status' => $type,
					'message' => $msg,
					);
				} else {
					$result = array(
					'status' => $type,
					'message' => $msg,
					);
				}
				echo json_encode($result);
				exit();
			} else {
				$message = $msg;
				set_message($type, $message);
			}
		}
		redirect('admin/masters/categories');
	}
	public function edit_category($id = NULL)
	{
		$data['title'] = lang('Edit') . ' ' . lang('category');
		if (!empty($id) && is_numeric($id)) {
			$data['active'] = 2;
			$edited = can_action('168', 'edited');
			if (!empty($edited)) {
				$data['subcategory_info'] = $this->masters_model->check_by(array('id' => $id), 'tbl_customer_categories');
			}
		} else {
			$data['active'] = 1;
		}
		$data['subview'] = $this->load->view('admin/masters/category_list', $data, TRUE);
		$this->load->view('admin/_layout_main', $data); //page load
	}
	
	public function delete_category($id)
	{
		$product_info = $this->masters_model->check_by(array('id' => $id), 'tbl_customer_categories');
		$activity = array(
		'user' => $this->session->userdata('user_id'),
		'module' => 'customer_category',
		'module_field_id' => $id,
		'activity' => 'category_deleted',
		'icon' => 'fa-circle-o',
		'value1' => $product_info->name
		);
		$this->masters_model->_table_name = 'tbl_activities';
		$this->masters_model->_primary_key = 'activities_id';
		$this->masters_model->save($activity);

		$this->masters_model->_table_name = 'tbl_customer_categories';
		$this->masters_model->_primary_key = 'id';
		$this->masters_model->delete($id);
		$type = 'success';
		$message = lang('category_deleted');
		echo json_encode(array("status" => $type, 'message' => $message));
		exit();
	}
	public function subcategories($id = NULL)
	{
		$data['title'] = lang('all') . ' ' . lang('subcategories');
		if (!empty($id) && is_numeric($id)) {
			$data['active'] = 2;
			$edited = can_action('169', 'edited');
			if (!empty($edited)) {
				$data['subcategory_info'] = $this->masters_model->check_by(array('id' => $id), $_REQUEST);
			}
		} else {
			$data['active'] = 1;
		}
		$data['categories'] = $this->items_model->get_all_lead_categories();
		$data['subview'] = $this->load->view('admin/masters/subcategory_list', $data, TRUE);
		$this->load->view('admin/_layout_main', $data); //page load
	}
	public function subcategoryList()
	{
		if ($this->input->is_ajax_request()) {
			$this->load->model('datatables');
			$this->datatables->table = 'tbl_customer_subcategories';
			$this->datatables->join_table = array('tbl_customer_categories');
			$this->datatables->join_where = array('tbl_customer_subcategories.cust_cat_id=tbl_customer_categories.id');
			$this->datatables->column_order = array('tbl_customer_subcategories.name', 'tbl_customer_categories.name  category', 'tbl_customer_subcategories.status');
			$this->datatables->column_search = array('tbl_customer_subcategories.name', 'tbl_customer_categories.name  category', 'tbl_customer_subcategories.status');
			
			$this->datatables->order = array('tbl_customer_subcategories.id' => 'desc');
			//$where = "type = 'Electricity'";

			$fetch_data = $this->datatables->get_subcategories();
		//	print_r($fetch_data);die;
			$data = array();

			$edited = can_action('170', 'edited');
			$deleted = can_action('170', 'deleted');
			foreach ($fetch_data as $_key => $v_items) {
				$action = null;
				$sub_array = array();
				$sub_array[] = $v_items->name;
				$sub_array[] = $v_items->category;
				$sub_array[] = $v_items->status;


				if (!empty($edited)) {
					$action .= btn_edit('admin/masters/edit_subcategory/' . $v_items->id) . ' ';
				}
				if (!empty($deleted)) {
					$action .= ajax_anchor(base_url("admin/masters/delete_subcategory/$v_items->id"), "<i class='btn btn-xs btn-danger fa fa-trash-o'></i>", array("class" => "", "title" => lang('delete'), "data-fade-out-on-success" => "#table_" . $_key)) . ' ';
				}
				$sub_array[] = $action;
				$data[] = $sub_array;
			}
			render_table($data);
		} else {
			redirect('admin/dashboard');
		}
	}

	public function saved_subcategories($id = NULL)
	
	{
		$created = can_action('170', 'created');
		$edited = can_action('170', 'edited');

		if (!empty($created) && !empty($edited)) {

			if (!empty($id) && $id == 'inline') {
				$id = null;
				$inline = true;
			}
			$this->masters_model->_table_name = 'tbl_customer_subcategories';
			$this->masters_model->_primary_key = 'id';

			$data = $this->masters_model->array_from_post(array('name','cust_cat_id', 'status'));
			// update root category
			$where = array('name' => $data['name'], 'cust_cat_id' => $data['cust_cat_id'],'status' => $data['status']);
			// duplicate value check in DB
			if (!empty($id)) {
				// if id exist in db update data
				$id = array('id !=' => $id);
			} else {
				// if id is not exist then set id as null
				$id = null;
			}
			// check whether this input data already exist or not
			$check_items = $this->masters_model->check_update('tbl_customer_subcategories', $where, $id);
			if (!empty($check_items)) {
				// if input data already exist show error alert
				// massage for user
				$type = 'error';
				$msg = "<strong style='color:#000'>" . $data['name'] . '</strong>  ' . lang('already_exist');
			} else {
				// save and update query
				$permission = $this->input->post('permission', true);

				$return_id = $this->masters_model->save($data, $id);

				save_custom_field(19, $id);

				if (!empty($id)) {
					$id = $id;
					$action = 'update_subcategory';
					$msg = lang('update_subcategory');
				} else {
					$id = $return_id;
					$action = 'save_subcategory';
					$msg = lang('save_subcategory');
				}
				$activity = array(
				'user' => $this->session->userdata('user_id'),
				'module' => 'customer_subcategory',
				'module_field_id' => $id,
				'activity' => $action,
				'icon' => 'fa-circle-o',
				'value1' => $data['name']
				);
				$this->masters_model->_table_name = 'tbl_activities';
				$this->masters_model->_primary_key = 'activities_id';
				$this->masters_model->save($activity);
				// messages for user
				$type = "success";
			}
			if (!empty($inline)) {
				if (!empty($return_id)) {
					$result = array(
					'id' => $id,
					'name' => $data['name'],
					'status' => $type,
					'message' => $msg,
					);
				} else {
					$result = array(
					'status' => $type,
					'message' => $msg,
					);
				}
				echo json_encode($result);
				exit();
			} else {
				$message = $msg;
				set_message($type, $message);
			}
		}

		redirect('admin/masters/subcategories');
	}
	public function edit_subcategory($id = NULL)

	{
		$data['title'] = lang('Edit') . ' ' . lang('category');
		if (!empty($id) && is_numeric($id)) {
			$data['active'] = 2;
			$edited = can_action('168', 'edited');
			if (!empty($edited)) {
				$data['subcategory_info'] = $this->masters_model->check_by(array('id' => $id), 'tbl_customer_subcategories');
			}
		} else {
			$data['active'] = 1;
		}
		$data['categories'] = $this->items_model->get_all_lead_categories();
		$data['subview'] = $this->load->view('admin/masters/subcategory_list', $data, TRUE);
		$this->load->view('admin/_layout_main', $data); //page load

	}

	public function delete_subcategory($id)
	{
		$product_info = $this->masters_model->check_by(array('id' => $id), 'tbl_customer_subcategories');
		$activity = array(
		'user' => $this->session->userdata('user_id'),
		'module' => 'customer_subCategory',
		'module_field_id' => $id,
		'activity' => 'subcategory_deleted',
		'icon' => 'fa-circle-o',
		'value1' => $product_info->name
		);
		$this->masters_model->_table_name = 'tbl_activities';
		$this->masters_model->_primary_key = 'activities_id';
		$this->masters_model->save($activity);

		$this->masters_model->_table_name = 'tbl_customer_subcategories';
		$this->masters_model->_primary_key = 'id';
		$this->masters_model->delete($id);
		$type = 'success';
		$message = lang('category_deleted');
		echo json_encode(array("status" => $type, 'message' => $message));
		exit();
	}


}
